// ContentsRestoreDlg.cpp : implementation file
//

#include "stdafx.h"
#include "khan.h"
#include "ContentsRestoreDlg.h"
#include "FileDialog2.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CContentsRestoreDlg dialog


CContentsRestoreDlg::CContentsRestoreDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CContentsRestoreDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CContentsRestoreDlg)
	m_strRestore = _T("");
	m_strMessage = _T("");
	m_strFile = _T("");
	//}}AFX_DATA_INIT
}


void CContentsRestoreDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CContentsRestoreDlg)
	DDX_Control(pDX, IDC_BUTTON_BROWSE, m_ctrlBrowseButton);
	DDX_Control(pDX, IDC_BUTTON_RESTORE_CONTENTS, m_ctrlRestoreButton);
	DDX_Control(pDX, IDC_EDIT_FILE, m_ctrlFile);
	DDX_Text(pDX, IDC_EDIT_FILE, m_strFile);
	DDX_Text(pDX, IDC_STATIC_RESTORE, m_strRestore);
	DDX_Text(pDX, IDC_STATIC_MSG, m_strMessage);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CContentsRestoreDlg, CDialog)
	//{{AFX_MSG_MAP(CContentsRestoreDlg)
	ON_BN_CLICKED(IDC_BUTTON_BROWSE, OnButtonBrowse)
	ON_BN_CLICKED(IDC_BUTTON_RESTORE_CONTENTS, OnRestoreContents)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CContentsRestoreDlg message handlers

BOOL CContentsRestoreDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// Set initial header string.
	int nTopics = 0;
	CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
	if(!pDoc)
	{
		m_ctrlRestoreButton.EnableWindow(FALSE);
		m_ctrlBrowseButton.EnableWindow(FALSE);
		UpdateData(FALSE);
		return TRUE;
	}

	nTopics = pDoc->GetTree().NumTopics();
	m_strRestore.Format(_T("Restore Contents from a backup file.\n")
					    _T("The current Contents contains %d Topics."), nTopics);

	// Lets not restore if other threads running.
	if(pDoc->m_videoPlaying		||
	   pDoc->m_hDownloadThread	||
	   pDoc->m_hAppUpdateThread ||
	   pDoc->m_hUpdateThread	)
	{
		m_strMessage = _T("Please stop any Video playback, Updates, or Downloads first.");
		m_ctrlRestoreButton.EnableWindow(FALSE);
		m_ctrlBrowseButton.EnableWindow(FALSE);
		UpdateData(FALSE);
		return TRUE;
	}

	// Set initial message.
	m_strMessage = _T("Press the Browse Button to locate a Contents backup file to restore.");
	UpdateData(FALSE);

	// Disable button till user browses.
	m_ctrlRestoreButton.EnableWindow(FALSE);	

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

// User browse for a backup file to Restore.
void CContentsRestoreDlg::OnButtonBrowse() 
{
	// Set dlg.
	CString kdcExtension = _T("kdc");
	CString filters = _T("KD Contents (*.kdc)|*.kdc||");
	CFileDialog2 dlg(TRUE, 
					kdcExtension, 
					_T("KhanContents_01.kdc"),
					OFN_HIDEREADONLY	| 
					OFN_OVERWRITEPROMPT | 
					OFN_NOCHANGEDIR,		// no change cur dir.
					filters,
					NULL );
	dlg.m_ofn.lpstrTitle = _T("Select Contents Backup file");

	// Call.
	if(dlg.DoModal() == IDOK)
	{
		CString path = dlg.GetPathName(); // C:\MyBackups\KhanContents_01.kdc

		// Verify .kdc extension - should be ok as dlg sets.
		CString s = path.Right(4);
		CString s2 = _T(".");
		s2 += kdcExtension;
		if(s != s2)
		{
			// Set message, button, and bail.
			m_strMessage.Format(_T("%s\nhas no .kdc file extension.\n")
								_T("Please enter a new file path."), path);
			m_ctrlRestoreButton.EnableWindow(FALSE);
			UpdateData(FALSE);
			return;
		}

		// Read the contents into a temp tree.
		CTopicTree tempTree;
		if(!tempTree.Read(path))
		{
			// Error
			//
			// Clear any text.
			//m_strFile.Empty();

			// Set initial message.
			m_strMessage = _T("Backup File invalid.\n")
						   _T("Press the Browse Button to locate a valid Contents backup file to restore.");
			UpdateData(FALSE);

			// Disable button till user re-browses.
			m_ctrlRestoreButton.EnableWindow(FALSE);

			UpdateData(FALSE);

		}

		// Count Topics.
		int nTopics = tempTree.NumTopics();

		// Cleam up a bit.
		tempTree.Zero();

		// Update member file path.
		m_strFile = path;

		// Update message.
		m_strMessage.Format(_T("Selected Contents Backup file contains %d Topics.\n")
							_T("Press the Restore Button to continue."), nTopics);

		// Enable button.
		m_ctrlRestoreButton.EnableWindow(TRUE);

		// Set controls.
		UpdateData(FALSE);
	}

	// User canceled the file dialog. Set default dlg.
	else
	{
		// Clear any text.
		m_strFile.Empty();

		// Set initial message.
		m_strMessage = _T("Press the Browse Button to locate a Contents backup file to restore.");
		UpdateData(FALSE);

		// Disable button till user re-browses.
		m_ctrlRestoreButton.EnableWindow(FALSE);

		UpdateData(FALSE);
	}	
}

// User pressed restore button.
void CContentsRestoreDlg::OnRestoreContents() 
{
	// Need CDoc.
	CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
	if(!pDoc)
	{
		m_strMessage = _T("Internal error code 1 - Contents restore failed.");
		m_ctrlRestoreButton.EnableWindow(FALSE);
		UpdateData(FALSE);
		return;
	}

	// Lets not restore if other threads running.
	if(pDoc->m_videoPlaying		||
	   pDoc->m_hDownloadThread	||
	   pDoc->m_hAppUpdateThread ||
	   pDoc->m_hUpdateThread	)
	{
		m_strMessage = _T("Please stop any Video playback, Updates, or Downloads first.");
		m_ctrlRestoreButton.EnableWindow(FALSE);
		UpdateData(FALSE);
		return;
	}

	// Copy backup file to AppData folder.
	// Note KhanDB.dat is read only at start up,
	// and written only during Updates.
	//
	// Get KhanDB path - same as CTopicTree::Write().
	CString toPath = ((CApp*)AfxGetApp())->GetAppDataFolder();
	toPath += _T("\\KhanDB.dat");

	// Restore file path should be set - see OnButtonBrowse().
	CString fromPath = m_strFile;

	// Attempt copy overwrite.
	if(CopyFile(fromPath, toPath, FALSE))
	{
		// Copied ok - update message.
		m_strMessage = _T("Contents restored successfully!");

		// Disable restore button.
		m_ctrlRestoreButton.EnableWindow(FALSE);

		UpdateData(FALSE);

		// Update the Contents.
		if(!pDoc->UpdateRestoredContents())
		{
			// MFC Error.
			m_strMessage = _T("Internal error code 2 - Contents restore failed.");
			m_ctrlRestoreButton.EnableWindow(FALSE);
			UpdateData(FALSE);
			return;
		}
	}

	// Error
	else
	{
		DWORD err = GetLastError();

		// Format last error.
		LPVOID lpMsgBuf;
		FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
					  FORMAT_MESSAGE_FROM_SYSTEM	 |
					  FORMAT_MESSAGE_IGNORE_INSERTS,
					  NULL,
					  err,
					  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
					  (LPTSTR) &lpMsgBuf,
					  0,
					  NULL);

		// Format err msg.
		m_strMessage.Format(_T("Contents Restore failed.\n%s"), lpMsgBuf);
		m_strMessage.TrimRight();

		// Clean up.
		LocalFree(lpMsgBuf);

		// Disable restore button.
		m_ctrlRestoreButton.EnableWindow(FALSE);

		UpdateData(FALSE);
	}
}
