// DeleteOrphans.cpp : implementation file
//

#include "stdafx.h"
#include "khan.h"
#include "DeleteOrphans.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDeleteOrphans dialog


CDeleteOrphans::CDeleteOrphans(CWnd* pParent /*=NULL*/)
	: CDialog(CDeleteOrphans::IDD, pParent)
{
	//{{AFX_DATA_INIT(CDeleteOrphans)
	m_strStatic1 = _T("");
	//}}AFX_DATA_INIT
}


void CDeleteOrphans::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDeleteOrphans)
	DDX_Control(pDX, IDC_BUTTON_DELETE, m_ctrlDelete);
	DDX_Control(pDX, IDC_LIST_ORPANS, m_ctrlList);
	DDX_Text(pDX, IDC_STATIC1, m_strStatic1);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDeleteOrphans, CDialog)
	//{{AFX_MSG_MAP(CDeleteOrphans)
	ON_BN_CLICKED(IDC_BUTTON_DELETE, OnButtonDelete)
	ON_LBN_SELCHANGE(IDC_LIST_ORPANS, OnSelchangeListOrpans)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDeleteOrphans message handlers

BOOL CDeleteOrphans::OnInitDialog() 
{
	// Profile release - 35 ms for 10 orphans in 500 files.

	// Base class.
	CDialog::OnInitDialog();

	// Disable till selected.
	m_ctrlDelete.EnableWindow(FALSE);

	// Get CDoc.
	CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
	if(!pDoc)
		return TRUE;
	CTopicTree & tree = pDoc->GetTree();

	// Prepare video Array.
	m_videoAr.RemoveAll();
	m_videoAr.SetSize(0, 32);
	CVideo video;

	// File vars.
	WIN32_FIND_DATA	FindData;
	HANDLE	hFiles;
	CString fileName;

	// File size calc.
	ULARGE_INTEGER	fileSz;
	__int64 nFileSz = 0;

	// Video store
	CString vidStore = pDoc->m_videoStorePath;
	vidStore += _T("\\");
	CString dir = vidStore;
	dir += _T("*");

	// Get orphans.
	if((hFiles = FindFirstFile(dir, &FindData)) != INVALID_HANDLE_VALUE) //0xffffffff NULL)//
	{
		while(FindNextFile(hFiles, &FindData))
		{
			// Watch for this or other directories.
			if(FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
				continue;

			// Get the file name.
			fileName = FindData.cFileName;

			// Ensure an mp4.
			if(fileName.Find(_T("mp4"), fileName.GetLength() - 3) == -1)
				continue;

			// If in the db.
			if(tree.VideoFileUsed(fileName))
				continue;

			// Get file size.
			fileSz.LowPart = FindData.nFileSizeLow; 
			fileSz.HighPart = FindData.nFileSizeHigh; 
			nFileSz = (__int64)fileSz.QuadPart;

			// Round up to next KB
			if(nFileSz % 1024)
				nFileSz = nFileSz / 1024 + 1;
			else 
				nFileSz = nFileSz / 1024;

			// Add to orphans list. 
			// Use CVideo as a storage struct.
			video.Zero();
			video.mp4 = fileName;
			video.duration.Format(_T("%d"), (int)nFileSz);
			m_videoAr.Add(video);
		}
		FindClose(hFiles);
	}
	// No video storage
	else
	{
		m_strStatic1 = _T("No video storage found");
		UpdateData(FALSE);
		return TRUE;
	}

	// Get count.
	int nVideos = m_videoAr.GetSize();

	// If no videos.
	if(!nVideos)
	{
		m_strStatic1 = _T("No unused video files found.");
		UpdateData(FALSE);
		return TRUE;
	}

	// Sort.
	m_videoAr.SortVideoDurationAscend();

	// Add videos to list box.
	SetListBoxItems();

	// Set Header text. Also in OnButtonDelete().
	m_strStatic1.Format(_T("Select for deletion any of the %d unused video ")
						_T("files listed below.\n")
						_T("Note:\tVideo file deletion cannot be undone.\n")
						_T("\tAdministrative privileges may be required."),
							nVideos);
	// Update ctrls.
	UpdateData(FALSE);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

// Delete button pressed.
void CDeleteOrphans::OnButtonDelete() 
{
	int nItems = m_ctrlList.GetSelCount();

	// Should not occure.
	if(nItems < 1)
	{
		m_strStatic1 = _T("Program error - selection count.");
		m_ctrlDelete.EnableWindow(FALSE);
		UpdateData(FALSE);
		return;
	}
	CArray<int, int> selItems;
	CString strItem, errMsg;
	int index, nDeleted = 0;

	// Get the array elements of all the selected items.
	selItems.SetSize(nItems);
	m_ctrlList.GetSelItems(nItems, selItems.GetData());

	// Delete each selected video file.
	for(int i = 0; i < nItems; i++)
	{
		// Get list item text.
		index = selItems[i];
		m_ctrlList.GetText(index, strItem);

		// Delete the file and remove the CVideo 
		// from the m_videoAr member.
		if(!DeleteVideoFile(strItem, errMsg))
		{
			m_strStatic1 = errMsg;
			m_ctrlList.SetCurSel(-1);
			m_ctrlDelete.EnableWindow(FALSE);
			UpdateData(FALSE);
			return;
		}
		nDeleted ++;
	}

	// If nothing was deleted.
	if(nItems && !nDeleted)
	{
		m_strStatic1 = _T("No files deleted.");
		m_ctrlList.SetCurSel(-1);
		m_ctrlDelete.EnableWindow(FALSE);
		UpdateData(FALSE);
		return;
	}

	// Redraw the list.
	int nVids = m_videoAr.GetSize();

	// Update header text
	if(nVids)
		m_strStatic1.Format(_T("Select for deletion any of the %d unused video ")
							_T("files listed below.\n")
							_T("Note:\tVideo file deletion cannot be undone.\n")
							_T("\tAdministrative privileges may be required."),
								nVids);
	else
		m_strStatic1 = _T("No unused video files found.");

	// Add any videos to list box.
	SetListBoxItems();

	// Disable till selected.
	m_ctrlDelete.EnableWindow(FALSE);

	// Update ctrls.
	UpdateData(FALSE);	
}

// Delete a video.
BOOL CDeleteOrphans::DeleteVideoFile(CString & listStr, CString & errMsg)
{
	// Need CDoc.
	CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
	if(!pDoc)
	{
		errMsg = _T("Program error - no document.");
		return FALSE;
	}

	// Loose the size.
	CString fileName = listStr;
	int del = fileName.Find(_T('\t'), 0);
	if(del == -1)
	{
		errMsg = _T("Program error - string conversion.");
		return FALSE;
	}
	fileName.Delete(0, del + 1);

	// Find and delete.
	int sz = m_videoAr.GetSize();

	for(int i = 0; i < sz; i++)
	{
		if(m_videoAr[i].mp4 == fileName)
		{
			// Get path.
			CString path = pDoc->m_videoStorePath;
			path += _T("\\");
			path += fileName;

			// Delete the file.
			if(!DeleteFile(path))
			{
				DWORD err = GetLastError();

				// Format last error.
				LPVOID lpMsgBuf;
				FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
							  FORMAT_MESSAGE_FROM_SYSTEM	 |
							  FORMAT_MESSAGE_IGNORE_INSERTS,
							  NULL,
							  err,
							  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
							  (LPTSTR) &lpMsgBuf,
							  0,
							  NULL);
				errMsg.Format(_T("Could not delete video file.\n%s"), lpMsgBuf);

				// Clean up.
				LocalFree(lpMsgBuf);

				// Bail
				return FALSE;
			}

			// Remove list string from array.
			m_videoAr.RemoveAt(i);

			// Ok.
			return TRUE;
		}
	}
	// Error.
	errMsg = _T("Program error - no files in array.");
	return FALSE;
}

// Set the list box with m_videoAr data.
void CDeleteOrphans::SetListBoxItems()
{
	// Empty.
	m_ctrlList.ResetContent();

	// Get video count.
	int nVideos = m_videoAr.GetSize();

	// If no videos.
	if(!nVideos)
		return;

	// Add videos to list box.
	CString str;
	for(int i = 0; i < nVideos; i++)
	{
		str.Format(_T("%s \t%s"), m_videoAr[i].duration, m_videoAr[i].mp4);
		m_ctrlList.AddString(str);
	}

	// Find the longest string in the list box
	// and set horz extents.
	CSize   sz;
	int     dx = 0;
	CDC *   pDC = m_ctrlList.GetDC();
	CFont * pFont = GetFont();					// Dialog font.
	CFont * old = pDC->SelectObject(pFont);		// Must select font.
	int nItems = m_ctrlList.GetCount();
	for(int i = 0; i < nItems; i++)
	{
		m_ctrlList.GetText(i, str);
		sz = pDC->GetTextExtent(str);

		if(sz.cx > dx)
			dx = sz.cx;
	}
	pDC->SelectObject(old);
	m_ctrlList.ReleaseDC(pDC);
	m_ctrlList.SetHorizontalExtent(dx + 5);	// Add a few pix.
}


// List box changing.
void CDeleteOrphans::OnSelchangeListOrpans() 
{
	// Enable disable button.
	int nItems = m_ctrlList.GetSelCount();
	if(nItems)
		m_ctrlDelete.EnableWindow(TRUE);
	else
		m_ctrlDelete.EnableWindow(FALSE);
	
}
