
/*	Copyright (C) G. Jespersen
	See Licence.txt.*/

// Doc.cpp : implementation of the CDoc class
//
// 
// File not compatable with VLC version
//

#include "stdafx.h"
#include "Khan.h"
#include "MainFrm.h"
#include "Doc.h"
#include "KhanInet.h"
#include "JsonParse.h"
#include "VideoStoreDlg.h"
#include "FontSetup.h"
#include "AppUpdateDlg.h"
#include "DeleteOrphans.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// App update.
#define NEW_EXE_STR _T("Khan_new.exe")
#define OLD_EXE_STR _T("Khan.exe.old")
#define CUR_EXE_STR _T("Khan.exe")
#define KDPATCH_STR _T("KDPatch.exe")

/////////////////////////////////////////////////////////////////////////////
// CDoc

IMPLEMENT_DYNCREATE(CDoc, CDocument)

BEGIN_MESSAGE_MAP(CDoc, CDocument)
	//{{AFX_MSG_MAP(CDoc)
	ON_COMMAND(ID_SETUP_VIDEO_STORE, OnSetupVideoStore)
	ON_COMMAND(ID_HELP_KHAN, OnHelpKhan)
	ON_COMMAND(ID_SETUP_UPDATE, OnApplicationUpdate)
	ON_COMMAND(ID_HISTORY_LAST_VIDEO, OnHistoryLastVideo)
	ON_COMMAND(ID_HISTORY_BACKUP, OnHistoryBackup)
	ON_COMMAND(ID_HISTORY_RESTORE, OnHistoryRestore)
	ON_COMMAND(ID_HISTORY_MERGE, OnHistoryMerge)
	ON_COMMAND(ID_HISTORY_REPAIR, OnHistoryRepair)
	ON_UPDATE_COMMAND_UI(ID_HISTORY_LAST_VIDEO, OnUpdateHistoryLastVideo)
	ON_UPDATE_COMMAND_UI(ID_HISTORY_BACKUP, OnUpdateHistoryBackup)
	ON_UPDATE_COMMAND_UI(ID_HISTORY_RESTORE, OnUpdateHistoryRestore)
	ON_UPDATE_COMMAND_UI(ID_HISTORY_REPAIR, OnUpdateHistoryRepair)
	ON_UPDATE_COMMAND_UI(ID_HISTORY_MERGE, OnUpdateHistoryMerge)
	ON_COMMAND(ID_TOOLS_ORPHAN_VIDEOS, OnToolsOrphanVideos)
	ON_COMMAND(ID_TOOLS_LIST_FILES, OnToolsListFiles)
	ON_UPDATE_COMMAND_UI(ID_TOOLS_ORPHAN_VIDEOS, OnUpdateToolsOrphanVideos)
	ON_UPDATE_COMMAND_UI(ID_TOOLS_LIST_FILES, OnUpdateToolsListFiles)
	ON_UPDATE_COMMAND_UI(ID_SETUP_VIDEO_STORE, OnUpdateSetupVideoStore)
	//}}AFX_MSG_MAP

	// Facilitate message routing to Dialog Bar.
	ON_BN_CLICKED(IDC_BUTTON_MUTE, OnMute)
	ON_BN_CLICKED(IDC_BUTTON_PLAY_PAUSE, OnPlayPause)
	ON_BN_CLICKED(IDC_BUTTON_STOP, OnStop)
	ON_BN_CLICKED(IDC_BUTTON_LEFT_PANE, OnHideLeftPane)


END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDoc construction/destruction

CDoc::CDoc()
{
	TRACE(_T("constructing CDoc\n"));

	// Program start up.
	m_startUp = TRUE;

	// Temp file path for updating
	m_tmpUpdateFilePath = ((CApp *)AfxGetApp())->GetAppDataFolder();
	m_tmpUpdateFilePath += _T("\\temp.dat");
	m_tmpUpdateFileSz = 0;

	// User Settings.
	//
	// Video Download Storage folder.
	m_videoStorePath = _T("C:\\Khan_video_folder");

	// View font.
	ZeroMemory(&m_viewLf, sizeof(LOGFONT));
	m_viewLf.lfHeight = -22;						// pixels - cell height
	m_viewLf.lfWeight = FW_NORMAL;					// FW_NORMAL = 400
	m_viewLf.lfQuality = PROOF_QUALITY | 
						 ANTIALIASED_QUALITY;
	//wcscpy(m_viewLf.lfFaceName, _T("Arial"));		// Lucida Console, Lucida Sans Unicode
	StringCbCopy(m_viewLf.lfFaceName, 6 * sizeof(TCHAR), _T("Arial")); 

	// Tree font.
	ZeroMemory(&m_treeLf, sizeof(LOGFONT));
	m_treeLf.lfHeight = -18;						// pixels - cell height. Def dlg -16
	m_treeLf.lfWeight = 500;						// FW_MEDIUM = 500
	m_treeLf.lfQuality = PROOF_QUALITY | 
						 ANTIALIASED_QUALITY;
	//wcscpy(m_treeLf.lfFaceName, _T("Arial"));		// Arial
	StringCbCopy(m_viewLf.lfFaceName, 6 * sizeof(TCHAR), _T("Arial")); // LF_FACESIZE 32 chars

	// Get any saved settings.
	ReadUserData();

	// Set CApp handle.
	((CApp *)AfxGetApp())->m_pDoc = this;

	// Khan Update worker Thread
	m_hUpdateThread = NULL;
	m_curNumTopics = 0;
	m_updateThreadParam.Zero();

	// Download Videos Worker thread.
	m_hDownloadThread = NULL;
	m_downloadThreadParam.Zero();

	// Application Update Thread
	m_hAppUpdateThread = NULL;
	m_appUpdateThreadParam.Zero();
	m_curVersion.Empty();
	m_tempUpdatePath.Empty();
	m_appUpdateInstalled = FALSE;

	// Inet Stream Test Worker thread. 12/13/14
//	m_hStreamTestThread = NULL;
//	m_streamTestThreadParam.Zero();

	// Video Playback ds
	m_videoPlaying = FALSE;		// set by View

	// Get any saved History.
	m_history.Read();

	// Topic Selection data.
	m_hCurTopic = NULL;
	m_nVideos = 0;
	m_nTotalVideoTime = 0; // seconds
	m_nChildTopics = 0;

	// Video Selection data.
	m_hCurVideo = NULL;

	// Index of CTabView tab curently displayed.
	m_curTabViewIndex = PAGE_CONTENTS; // def in Doc.h

	// 10/04/2017 search
	// See: CTreeDialog::OnSetSelSearch() & CSearchDialog::OnSearchGoBtn() 
	m_searchNodes.RemoveAll();
	m_searchVideoIndex = -1;
}

CDoc::~CDoc()
{
	// Set CApp handle.
	CApp * pApp = (CApp *)AfxGetApp();
	if(pApp)
		pApp->m_pDoc = NULL;

}

// Program Shutting down.
void CDoc::OnCloseDocument() 
{
	TRACE(_T("in CDoc OnCloseDocument - shutting down\n"));

	// Clean up any active worker threads.
	if(m_hUpdateThread)
		CancelUpdate();
	if(m_hDownloadThread)
		CancelVideoDownload();
	if(m_hAppUpdateThread)
		CancelAppUpdate();

	// Stop any video playback, and save a history,
	CDataView * pView = GetViewPtr();
	if(pView)
	{
		if(pView->IsVideoRunningOrPaused())
		{
			TRACE(_T("CDoc OnCloseDocument - stopping video playback\n"));
			pView->StopVideoPlayback();
		}
	}

	// Save user settings.
	WriteUserData();

	// Save Video History.
	m_history.Write();
	
	// Base class.
	CDocument::OnCloseDocument();
}

// Called from CDataView::StopVideoPlayback 
// when Playback is ending.
void CDoc::SetHistoryItem(UINT seconds) 
{
	TRACE(_T("in CDoc SetHistoryItem\n"));

	// Ensure valid video.
	if(m_curVideo.title.IsEmpty()		||
	   m_curVideo.topic.IsEmpty()		||
	   m_curVideo.duration.IsEmpty()	)
		return;

	// Bail if viewed for less than 10 seconds.
	if(seconds < 10)
		return;

	// Add / Update.
	CHistoryItem item;
	item.m_video = m_curVideo;
	item.m_lastView = CTime::GetCurrentTime();
	item.m_totalViewTime = seconds; //m_historyTotalTime;
	m_history.AddHistoryItem(item);

	// Update the Tree Dialog to reflect any new history marks.
	HWND hWnd = ((CApp *)AfxGetApp())->m_hwndTreeDlg;
	if(hWnd)
		PostMessage(hWnd, WM_USER_TREE_VID_UPDATE, 0, 0);
}

BOOL CDoc::OnNewDocument()
{
	TRACE(_T("in CDoc OnNewDocument()\n"));

	if(!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	// On start up, or after an update, set the tree.
	// See CDoc::UpdateFinished().
	if(m_startUp)
	{
		m_startUp = FALSE;
		// Zero.
		m_tree.Zero();

		// Get any saved khan data.
		m_tree.Read();

		int nTopics = m_tree.NumTopics(); // 109 topics
		if(nTopics)
		{
			HWND hwnd = ((CApp *)AfxGetApp())->m_hwndTreeDlg;

			// Update the tree dialog.
			if(hwnd)
				PostMessage(hwnd, WM_USER_DLG_UPDATE, 0, 0);
		}
	}
	return TRUE;
}

// Contents restore.
// Called from CContentsRestoreDlg::OnRestoreContents() only.
BOOL CDoc::UpdateRestoredContents()
{
	// Set flag.
	m_startUp = TRUE;

	// Call OnNewDocument() to read restored KhanDB.dat
	// and update the tree dialog.
	return OnNewDocument();
}

// Display HTML Help 
void CDoc::OnHelpKhan() 
{
	// Call chm in current directory.
	HtmlHelp(0,
			 _T("KhanHelp.chm"),
			 HH_DISPLAY_TOC,
			 NULL);
}

// Show last video viewed.
void CDoc::OnHistoryLastVideo() 
{
	// Playback must be stopped.
	if(m_videoPlaying)
		return;

	CVideo lastViewed;

	// Get the last.
	if(m_history.GetLastViewedVideo(lastViewed))
	{
		// Ensure a valid history item.
		if(!m_tree.VideoExists(lastViewed))
		{
			MessageBox(NULL, _T("The Last Viewed video is an invalid History\n")
							 _T("item. Please repair the History by using\n")
							 _T("the History / Repair menu item."), 
							 _T("Khan Desktop"), 
								MB_OK | MB_ICONINFORMATION);
			m_historyLastViewed.Zero();
			return;
		}

		// CTreeDialog::OnSetSelLastViewed handles.
		HWND hWnd = ((CApp *)AfxGetApp())->m_hwndTreeDlg;
		if(hWnd)
		{
			// Set member - CTreeDialog fetches.
			m_historyLastViewed = lastViewed;

			// Message dlg.
			::PostMessage(hWnd, WM_USER_TREE_SET_SEL, 0, 0);

			// Ensure left is showing.
			// Na - buggy - need initial size set better
			//hWnd = ((CApp*)AfxGetApp())->m_hwndSplitter;
			//if(hWnd) ::PostMessage(hWnd, WM_USER_HIDE_LEFT_PANE, 0, 0);	
		}
	}
	else
		MessageBox(NULL, _T("No Khan videos have been viewed."), _T("Khan Desktop"), 
						MB_OK | MB_ICONINFORMATION);	
}
// UI Update.
void CDoc::OnUpdateHistoryLastVideo(CCmdUI* pCmdUI) 
{
	UpdateHistoyMenu(pCmdUI);	
}

// Backup History.
void CDoc::OnHistoryBackup() 
{
	// Note info: Apparently CFileDlg saves the last opened path in:
	//	HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion
	//	\Explorer\ComDlg32\OpenSaveMRU

	// Set dlg.
	CString filters = _T("KD Viewing History (*.kdh)|*.kdh||");
	CFileDialog dlg(FALSE, 
					HIST_BACKUP_EXT, 
					_T("KDHistBk_01.kdh"),
					OFN_HIDEREADONLY	| 
					OFN_OVERWRITEPROMPT | 
					OFN_NOCHANGEDIR,		// no change cur dir.
					filters,
					NULL );
	dlg.m_ofn.lpstrTitle = _T("Save Viewing History Backup to");

	// Call.
	if(dlg.DoModal() == IDOK)
	{
		CString path = dlg.GetPathName(); // C:\Khan_video_folder\KDHistBK.kdh

		// Verify .kdh extension.
		CString s = path.Right(4);
		CString s2 = _T(".");
		s2 += HIST_BACKUP_EXT;
		if(s != s2)
		{
			s.Format(_T("%s\nhas no .kdh file extension.\nCannot save backup."), path);
			MessageBox(NULL, s, _T("Khan Desktop Application"), MB_OK | MB_ICONINFORMATION);
			return;
		}

		// Write backup.
		//
		// Info user success.
		if(m_history.BackupHistory(path))
		{
			int num = m_history.NumUniqueHistoryItems();
			s.Format(_T("Successfully backed up History containing %d records."), num);
			MessageBox(NULL, s, _T("Khan Desktop Application"), MB_OK | MB_ICONINFORMATION);
		}
		// Info user error.
		else
		{
			MessageBox(NULL, _T("History backup failed."), 
					   _T("Khan Desktop Application"), 
					   MB_OK | MB_ICONINFORMATION);
		}
	}	
}
// UI Update.
void CDoc::OnUpdateHistoryBackup(CCmdUI* pCmdUI) 
{
	UpdateHistoyMenu(pCmdUI);	
}

// Restore History.
void CDoc::OnHistoryRestore() 
{
	// Note info: Apparently CFileDlg saves the last opened path in:
	//	HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion
	//	\Explorer\ComDlg32\OpenSaveMRU

	// Set dlg.
	CString filters = _T("KD Viewing History (*.kdh)|*.kdh||");
	CFileDialog dlg(TRUE, 
					HIST_BACKUP_EXT, 
					NULL,
					OFN_HIDEREADONLY	| 
					OFN_OVERWRITEPROMPT | 
					OFN_NOCHANGEDIR		|		// no change cur dir.
					OFN_FILEMUSTEXIST	|
					OFN_PATHMUSTEXIST,
					filters,
					NULL );
	dlg.m_ofn.lpstrTitle = _T("Open Backup file for History Restore");	// m_ofn.lpstrInitialDir	

	// Call.
	if(dlg.DoModal() == IDOK)
	{
		CString path = dlg.GetPathName();

		// Verify .kdh extension.
		CString s = path.Right(4);
		CString s2 = _T(".");
		s2 += HIST_BACKUP_EXT;
		if(s != s2)
		{
			s.Format(_T("%s\nhas no .kdh file extension.\nCannot restore."), path);
			MessageBox(NULL, s, _T("Khan Desktop Application"), MB_OK | MB_ICONINFORMATION);
			return;
		}
		// Read backup.
		//
		// Restore successfull, update last viewed.
		if(m_history.RestoreHistory(path))
		{
			// Update any visable Tree Dialog video marks
			// to reflect the restored history.
			HWND hWnd = ((CApp *)AfxGetApp())->m_hwndTreeDlg;
			if(hWnd)
				PostMessage(hWnd, WM_USER_TREE_VID_UPDATE, 0, 0);
		}
	}	
}
// UI Update.
void CDoc::OnUpdateHistoryRestore(CCmdUI* pCmdUI) 
{
	// If playing, disable restore.
	if(m_videoPlaying)
		pCmdUI->Enable(FALSE);
	else
		pCmdUI->Enable(TRUE);
}

// Repair the current history.
void CDoc::OnHistoryRepair() 
{
	if(m_history.RepairHistory())
	{
		// Update the Tree Dialog.
		HWND hWnd = ((CApp *)AfxGetApp())->m_hwndTreeDlg;
		if(hWnd)
			PostMessage(hWnd, WM_USER_TREE_VID_UPDATE, 0, 0);
	}
}
// UI Update.
void CDoc::OnUpdateHistoryRepair(CCmdUI* pCmdUI) 
{
	UpdateHistoyMenu(pCmdUI);	
}

// Merge a History.
void CDoc::OnHistoryMerge() 
{
	// Note info: Apparently CFileDlg saves the last opened path in:
	//	HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion
	//	\Explorer\ComDlg32\OpenSaveMRU

	// Set dlg.
	CString filters = _T("KD Viewing History (*.kdh)|*.kdh||");
	CFileDialog dlg(TRUE, 
					HIST_BACKUP_EXT, 
					NULL,
					OFN_HIDEREADONLY	| 
					OFN_OVERWRITEPROMPT | 
					OFN_NOCHANGEDIR		|		// no change cur dir.
					OFN_FILEMUSTEXIST	|
					OFN_PATHMUSTEXIST,
					filters,
					NULL );
	dlg.m_ofn.lpstrTitle = _T("Open Backup file for History Merge");	// m_ofn.lpstrInitialDir	

	// Call.
	if(dlg.DoModal() == IDOK)
	{
		CString path = dlg.GetPathName();

		// Verify .kdh extension.
		CString s = path.Right(4);
		CString s2 = _T(".");
		s2 += HIST_BACKUP_EXT;
		if(s != s2)
		{
			s.Format(_T("%s\nhas no .kdh file extension.\nCannot merge."), path);
			MessageBox(NULL, s, _T("Khan Desktop Application"), MB_OK | MB_ICONINFORMATION);
			return;
		}
		// Merge the backup.
		//
		// Merge successfull, update last viewed.
		if(m_history.MergeHistory(path))
		{
			// Update any visable Tree Dialog video marks
			// to reflect the merged history.
			HWND hWnd = ((CApp *)AfxGetApp())->m_hwndTreeDlg;
			if(hWnd)
				PostMessage(hWnd, WM_USER_TREE_VID_UPDATE, 0, 0);
		}
	}	
}
// UI Update.
void CDoc::OnUpdateHistoryMerge(CCmdUI* pCmdUI) 
{
	UpdateHistoyMenu(pCmdUI);
}

// Update History menu handler for 3 items.
void CDoc::UpdateHistoyMenu(CCmdUI* pCmdUI)
{
	// If playing, disable.
	if(m_videoPlaying)
		pCmdUI->Enable(FALSE);
	else
	{
		// If have a history.
		if(m_history.NumHistoryItems())
		{
			pCmdUI->Enable(TRUE);
		}
		else
			pCmdUI->Enable(FALSE);
	}
}

// Facilitate dialog bar message routing.
void CDoc::OnMute() 
{	}
void CDoc::OnPlayPause() 
{	}
void CDoc::OnStop() 
{	}
void CDoc::OnHideLeftPane() 
{	}


/////////////////////////////////////////////////////////////////////////////
// CDoc serialization

void CDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CDoc diagnostics

#ifdef _DEBUG
void CDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG


/////////////////////////////////////////////////////////////////////////////
// CDoc commands


// From Wanderer - Prosise - ch. 11
// Called from CMainFrame::OnCmdMsg().
BOOL CDoc::RouteCmdToAllViews(CView *pView, UINT nID, int nCode, void *pExtra,
							  AFX_CMDHANDLERINFO *pHandlerInfo)
{
    POSITION pos = GetFirstViewPosition();

    while (pos != NULL)
	{
        CView * pNextView = GetNextView(pos);
        if (pNextView != pView)
		{
            if (pNextView->OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
                return TRUE;
        }
    }
    return FALSE;
}

// Get CDataView pointer.
CDataView * CDoc::GetViewPtr()
{
	// There's a Tab View, Splitter View, and Data View.
	POSITION pos = GetFirstViewPosition();
	while (pos != NULL)
	{
		CView * viewPtr = GetNextView(pos);

		// If CDataView.
		if(viewPtr->IsKindOf( RUNTIME_CLASS(CDataView) ))
		{			
			return (CDataView *)viewPtr;
		}
	}
	// Nothing.
	return NULL;
}

///////////////////////////////////////////////////////////////////////
// Khan Update Handlers.

// Called from CKhanDialog::OnButtonUpdate().
// Will cancel the Khan Update.
void CDoc::CancelUpdate()
{
	// Kill the worker thread.
	//
	// Thread will call CDoc::OnThreadAborted() 
	// via CMainFrame.
    m_updateEvent.SetEvent();
}

// Called from CKhanDialog::OnButtonUpdate().
//
// Makes api call : www.khanacademy.org/api/v1/playlists/library/list
//
void CDoc::UpdateKhanLibrary(HWND hDlgWnd,
							 BOOL sortRoot, BOOL sortChild, BOOL sortVideo)
{
	// Ensure not in use.
	if(m_hUpdateThread != NULL)
		return;

	// Prepare.
	m_updateEvent.ResetEvent();	// set canceling to nonsignaled

	// Get current num topics. Add some for new topics and for 
	// other processing actions. See KhanUpdateThread().
	m_curNumTopics = m_tree.NumTopics();
	m_curNumTopics += 50;

	// Set param member.
	CMainFrame * pFrame = (CMainFrame*)(AfxGetMainWnd());
	if(!pFrame)
		return;
	m_updateThreadParam.hwndMain	= pFrame->GetSafeHwnd();		// CMainFrame HWND
	m_updateThreadParam.hwndDlg		= hDlgWnd;						// CDialog HWND
	m_updateThreadParam.pEvent		= &m_updateEvent;				// CEvent
	m_updateThreadParam.topicTree	= &m_tempTopicTree;				// Temp Topic Tree
	m_updateThreadParam.path		= &m_tmpUpdateFilePath;			// Temp update file
	m_updateThreadParam.fileSz		= &m_tmpUpdateFileSz;			// Temp update file sz
	m_updateThreadParam.numTopics	= &m_curNumTopics;				// For progress ctrl
	m_updateThreadParam.sortRoot	= sortRoot;						// Sort options
	m_updateThreadParam.sortChild	= sortChild;
	m_updateThreadParam.sortVideo	= sortVideo;

	// Start worker thread in suspended state.
	CWinThread * pThread = AfxBeginThread(KhanUpdateThread, 
										  &m_updateThreadParam,
										  THREAD_PRIORITY_NORMAL, 
										  0, 
										  CREATE_SUSPENDED);
	// Copy handle to m_hInetThread member.
	::DuplicateHandle(GetCurrentProcess(),
					  pThread->m_hThread, 
					  GetCurrentProcess(), 
					  &m_hUpdateThread,
					  0, 
					  FALSE, 
					  DUPLICATE_SAME_ACCESS);
	// Start thread.
	pThread->ResumeThread();
}

// Called from CMainFrame::OnUpdateFinished() - WM_USER_UPDATE_FINISHED
void CDoc::UpdateFinished()
{
	// Shut down the Update worker thread.
	if(m_hUpdateThread)
	{
		::WaitForSingleObject(m_hUpdateThread, INFINITE);
		::CloseHandle(m_hUpdateThread);
		m_hUpdateThread = NULL;

		// Read updated data to m_tree, and update CTreeDlg.
		//
		// Note: calling m_tree.Read() directly seems buggy.
		// Perhaps CDocument is needed via OnNewDocument().
		m_startUp = TRUE;
		OnNewDocument();

		// Inform Update Dialog of update completion.
		PostMessage(m_updateThreadParam.hwndDlg, WM_USER_FINISHED, 0, 0);

		// Clean up thread vars.
		m_tempTopicTree.Zero();
		m_updateThreadParam.Zero();

		// Delete temp file for updating.
		CFileStatus status;
		if(CFile::GetStatus(m_tmpUpdateFilePath, status)) // static version for test.
		{
			// Save file size for future progress ctrl setup in KhanUpdateThread().
			if(status.m_size)
				m_tmpUpdateFileSz = (DWORD)status.m_size;

			// For debugging the parse routines, save a copy to C:\\Khan.dat.
			// See CDataView::OnTest().
			#ifdef _DEBUG
				CopyFile(m_tmpUpdateFilePath, _T("C:\\Khan.dat"), FALSE);
			#endif			

			// Delete. 
			try
			{
				CFile::Remove(m_tmpUpdateFilePath);
			}
			catch (CFileException* pEx)
			{			
				pEx->Delete();
			}
		}
	}
}

// Called from KhanUpdateThread() via user message to
// CMainFrame::OnUpdateAborted() - WM_USER_UPDATE_ABORTED.
void CDoc::UpdateAborted()
{
	// Shut down the Update worker thread.
	if(m_hUpdateThread)
	{
		::WaitForSingleObject(m_hUpdateThread, INFINITE);
		::CloseHandle (m_hUpdateThread);
		m_hUpdateThread = NULL;

		// Inform Dialog
		PostMessage(m_updateThreadParam.hwndDlg, WM_USER_FINISHED, 0, 0);

		// No post processing.

		// Clean up thread vars.
		m_tempTopicTree.Zero();
		m_updateThreadParam.Zero();

		// Delete temp file for updating.
		CFileStatus status;
		if(CFile::GetStatus(m_tmpUpdateFilePath, status)) // static version for test.
		{
			// For debugging the parse routines, save a copy to C:\\Khan.dat.
			// See CDataView::OnTest().
			#ifdef _DEBUG
				CopyFile(m_tmpUpdateFilePath, _T("C:\\Khan.dat"), FALSE);
			#endif			

			try
			{
				CFile::Remove(m_tmpUpdateFilePath);
			}
			catch (CFileException* pEx)
			{			
				pEx->Delete();
			}
		}
	}
}

// Global Khan Update Thread.
//
// Gets json data from the khan server with the request:
//  www.khanacademy.org/api/v1/playlists/library/list
//
// Parses and serializes the json data.
// Calls CDoc::ThreadFinished() to update the CDoc::m_tree member 
// and the CTreeDialog's Tree Control.
// 
UINT KhanUpdateThread(LPVOID pParam)
{
	// Copy param data.
	THREADPARAM param = *(THREADPARAM *) pParam;

	// Storage for binary json data for parsing.
	CString tempPath = *(CString *)param.path;

	// Return params - not used.
	CString errMsg;
	DWORD errCode = 0;

	// Calc dialogs progress step base on last update file size.
	// Note getting size from Khan server does not work.
	//
	// step = 4,252 on 9/12, 22,464 8/13, 14,469 on 11/13, 29,562 on 4/15
	//		  47,798 on 6/16, 730,000 on 5/17  (after / max)
 	//
	// 5/30/2017 changed max from 100 to 1000 - final step = 73425
	//  download = 70.02 mb, 4545 topics.
	// 10/02/2017 download = 75.37 mb, 5233 topics.
	// 10/04/2017 download = 80.23 mb, 5559 topics - final step = 90000
	int max = 1000;
	int pos = 0;
	int step = *param.fileSz;
	if(step < 90000000)			// if not initialized, bigger is better.			
		step = 90000000;
	step /= max;

	// Prepare dialog's progress ctrl.
	PostMessage(param.hwndDlg, WM_USER_PROGRESS_MAX, 0, (LPARAM)max);
	PostMessage(param.hwndDlg, WM_USER_PROGRESS_POS, 0, (LPARAM)pos);

	//////////////////////
	// Get KhanInet data.
	CKnanInet inet(param); 
	if(!inet.GetKhanLibraryData(tempPath, step))
	{
		// CKhanInet error.
		// Post thread abort message to CDoc via CMainFrame.
		PostMessage(param.hwndMain, WM_USER_UPDATE_ABORTED, 0, 0);
		return 1;
	}

	////////////////////////////////
	// Download successfull. Parse.
	//
	TRACE(_T("KhanInetLibraryThread begin parse\n"));

	// Prepare progress. 
	// Note progress is stepped for each topic parsed.
	//	numTopics = 159 on 9/12, 571 on 1/13, 1900 on 4/15, 
	//  4733 on 6/17, 5233 on 10/17, 5559 on 11/17 
	max = *param.numTopics;
	if(max < 6000)
		max = 6000;
	pos = 1;
	PostMessage(param.hwndDlg, WM_USER_PROGRESS_MAX, 0, (LPARAM)max);
	PostMessage(param.hwndDlg, WM_USER_PROGRESS_POS, 0, (LPARAM)pos);

	// Get message string for appending.
	CString msgStr;
	CApp * pApp = ((CApp*)AfxGetApp());
	pApp->GetThreadMessage(msgStr);
	msgStr += _T("\r\n");

	// Set dialog message.
	msgStr += _T("Processing Khan Data.\r\n");
	msgStr += _T("Please wait.\r\n");
	PostMessage(param.hwndDlg, WM_USER_STATUS_STRING, 0, 0);

	// Prepare.
	param.topicTree->Zero();

	// Parse into param's tree member.
	CParseJson json;
	if(!json.ParseLibrary(param.topicTree, tempPath,
						 param.hwndDlg, pos, errMsg))
	{
		// CKhanJson error. Info user.
		msgStr += _T("ERROR - processing Khan data.\r\n");
		msgStr += errMsg;
		msgStr += _T("\r\n");
		pApp->SetThreadMessage(msgStr);
		PostMessage(param.hwndDlg, WM_USER_STATUS_STRING, 0, 0);

		// Post thread abort msg to CDoc via CMainFrame.
		PostMessage(param.hwndMain, WM_USER_UPDATE_ABORTED, 0, 0);

		return 1;
	}

	// Count topics gotten. 
	int nTopics = param.topicTree->NumTopics();

	////////////////////////
	// Verify parse results
	//
	if(nTopics < 600			||	// 159 on 09/12, 763 08/13, 1900 05/15, 2664 05/16, 5500 11/17
//	   json.m_nErrorTopics		||	// Topics were lost
	   json.m_nNodeBraceError	)	// JSON brace errors
	{
		// Error.
		msgStr += _T("ERROR - Khan data corrupted.\r\n");
		pApp->SetThreadMessage(msgStr);
		PostMessage(param.hwndDlg, WM_USER_STATUS_STRING, 0, 0);

		// Post thread abort msg to CDoc via CMainFrame.
		PostMessage(param.hwndMain, WM_USER_UPDATE_ABORTED, 0, 0);

		// Dbg msg.
		TRACE(_T("KhanInetLibraryThread parsed data corrupted:\n"));
//		TRACE(_T("Topic errors: %d  Brace errors: %d Missing mp4: %d\n"),
//					json.m_nErrorTopics, json.m_nErrorBraces, json.m_nErrorVideos);
		return 1;
	}

	////////////////////////
	// Parsing successfull.
	TRACE(_T("KhanInetLibraryThread parse successfull\n"));
	CString s;

	// If user is aborting.
	if(::WaitForSingleObject(param.pEvent->m_hObject, 0) == WAIT_OBJECT_0) 
	{
		// Post Status msg.
		msgStr += _T("Canceling processing.\r\n");
		pApp->SetThreadMessage(msgStr);
		PostMessage(param.hwndDlg, WM_USER_STATUS_STRING, 0, 0);

		// Post thread abort msg to CMainFrame.
		PostMessage(param.hwndMain, WM_USER_UPDATE_ABORTED, 0, 0);

		// Bail.
		return 1;
	}

	// Finish
	//
	// Patch any Khan Api errors. New for v0.8.1
	//param.topicTree->PatchUpdatedTree(); moved up

	// Sort Topics.
	msgStr += _T("Sorting Topics.\r\n");
	pApp->SetThreadMessage(msgStr);
	PostMessage(param.hwndDlg, WM_USER_STATUS_STRING, 0, 0);
	pos += 5;
	PostMessage(param.hwndDlg, WM_USER_PROGRESS_POS, 0, (LPARAM)pos);
	param.topicTree->Sort(param.sortRoot, param.sortChild, param.sortVideo);

	// Serialize Topics.
	s.Format(_T("Saving %d Topics.\r\n"), nTopics);
	msgStr += s;
	pApp->SetThreadMessage(msgStr);
	PostMessage(param.hwndDlg, WM_USER_STATUS_STRING, 0, 0);
	pos += 5;
	PostMessage(param.hwndDlg, WM_USER_PROGRESS_POS, 0, (LPARAM)pos);
	param.topicTree->Write();

	// Finished. CDoc will read serialized data and 
	// update the Tree Dialog.
	//
	// Set Progress all the way.
	PostMessage(param.hwndDlg, WM_USER_PROGRESS_POS, 0, (LPARAM)max);

	// Done message
	msgStr += _T("Khan Update Successfull.\r\n");
	pApp->SetThreadMessage(msgStr);
	PostMessage(param.hwndDlg, WM_USER_STATUS_STRING, 0, 0);

	// Post thread finished msg to CDoc via CMainFrame.
	PostMessage(param.hwndMain, WM_USER_UPDATE_FINISHED, 0, 0);

	TRACE(_T("KhanInetLibraryThread - processing complete\n"));

	return 0;
}

///////////////////////////////////////////////////////////////////////
// Video Download Handlers.

// Check if a video is stored on disk,
BOOL CDoc::VideoOnDisk(CVideo & video)
{
	CString fileName;
	if(!video.GetShortFileName(fileName))
		return FALSE;;

	// Search for the file in the m_videoStorePath folder. 
	WIN32_FIND_DATA	FindData;
	HANDLE hFile;
	CString path = m_videoStorePath;
	path += _T('\\');
	path += fileName;
	hFile = FindFirstFile(path, &FindData);

	// If exists.
	if(hFile != INVALID_HANDLE_VALUE)
		return TRUE;

	return FALSE;
}

// Check if a video is in the download list.
BOOL CDoc::VideoForDownload(CVideo & video)
{
	int sz = m_vidDnLdAr.GetSize();
	for(int i = 0; i < sz; i++)
	{
		// Not good - compares vids topic etc.
		//if(m_vidDnLdAr[i] == video)

		// Compare mp4 members.
		if(m_vidDnLdAr[i].mp4 == video.mp4)
			return TRUE;
	}
	return FALSE;
}

// Remove a video from the download list.
void CDoc::RemoveVideoForDownload(CVideo & video)
{
	// If currently downloading, it's too late to remove.
	if(DownloadingVideos())
		return;

	int sz = m_vidDnLdAr.GetSize();
	for(int i = 0; i < sz; i++)
	{
		// Not good - compares vids topic etc.
		//if(m_vidDnLdAr[i] == video)

		// Compare mp4 members.
		if(m_vidDnLdAr[i].mp4 == video.mp4)
		{
			m_vidDnLdAr.RemoveAt(i);
			return;
		}
	}
}

// Remove all videos from the m_vidDnLdAr array
// that have been downloaded.
void CDoc::RemoveDownloadedVideos()
{
	int sz = m_vidDnLdAr.GetSize();
	CString fileName;

	// Loop backwards through the array, 
	// deleting from the bottom up.
	for(int i = sz - 1; i >= 0; i--)
	{
		CVideo & vid = m_vidDnLdAr[i];
		if(VideoOnDisk(vid))
			m_vidDnLdAr.RemoveAt(i);
	}
}

// Add a video to the download list.
void CDoc::AddVideoForDownload(CVideo & video)
{
	//
	// Note: videos may be added to the list during video download.
	// The dowload thread uses a copy of the array.
	//

	// Ensure no duplicates.
	int sz = m_vidDnLdAr.GetSize();
	for(int i = 0; i < sz; i++)
	{
		// NO - this allows dups as topic will always be different.
		// Note however CKnanInet::GetVideoFiles() overwrites any existing.
		//if(m_vidDnLdAr[i] == video)

		// Check file url / path for duplicates.
		// Note this allows duplicate titles.
		if(m_vidDnLdAr[i].mp4 == video.mp4)
			return;
	}
	// Add.
	m_vidDnLdAr.Add(video);
}

// Get the video at download list's index.
BOOL CDoc::GetVideoForDownload(int index, CVideo & video)
{
	int sz = m_vidDnLdAr.GetSize();
	if(index >= sz)
		return FALSE;
	video = m_vidDnLdAr[index];
	return TRUE;
}

// Remove all videos marked for download and
// update the Download Dialog and Tree Dialog.
void CDoc::RemoveAllVideosForDownload()
{
	// If currently downloading, it's too late to remove.
	if(DownloadingVideos())
		return;

	// Empty array.
	m_vidDnLdAr.RemoveAll();

	// Update the Download Dialog.
	HWND hWnd = ((CApp *)AfxGetApp())->m_hwndDownloadDlg;
	if(hWnd)
		PostMessage(hWnd, WM_USER_FINISHED, 0, 0);

	// Update the Tree Dialog.
	hWnd = ((CApp *)AfxGetApp())->m_hwndTreeDlg;
	if(hWnd)
		PostMessage(hWnd, WM_USER_TREE_VID_UPDATE, 0, 0);
}

// Called from CDownloadDialog::OnButtonDownload().
// Will cancel the video download.
void CDoc::CancelVideoDownload()
{
	// Kill the worker thread.
	//
	// Thread will call CDoc::ThreadAborted() 
	// via CMainFrame.
    m_downloadEvent.SetEvent();
}

// Called from CMainFrame::OnDownloadFinished() - WM_USER_DOWNLOAD_FINISHED
void CDoc::DownloadFinished()
{
	// Shut down the Download worker thread.
	if(m_hDownloadThread)
	{
		::WaitForSingleObject(m_hDownloadThread, INFINITE);
		::CloseHandle (m_hDownloadThread);
		m_hDownloadThread = NULL;

		// Remove videos downloaded to disk 
		// from the m_vidDnLdAr array.
		RemoveDownloadedVideos();

		// Clean the temp array.
		m_vidDnLdArTemp.RemoveAll();

		// Update the Download Dialog.
		PostMessage(m_downloadThreadParam.hwndDlg, WM_USER_FINISHED, 0, 0);

		// Update the Tree Dialog.
		HWND hWnd = ((CApp *)AfxGetApp())->m_hwndTreeDlg;
		if(hWnd)
			PostMessage(hWnd, WM_USER_TREE_VID_UPDATE, 0, 0);

		// Clean up thread vars.
		m_downloadThreadParam.Zero();
	}
}

// Called from CMainFrame::OnDownloadAborted() - WM_USER_DOWNLOAD_ABORTED
void CDoc::DownloadAborted()
{
	// Shut down the Download worker thread.
	if(m_hDownloadThread)
	{
		::WaitForSingleObject(m_hDownloadThread, INFINITE);
		::CloseHandle (m_hDownloadThread);
		m_hDownloadThread = NULL;

		// Remove videos downloaded to disk 
		// from the m_vidDnLdAr array.
		RemoveDownloadedVideos();

		// Update the Download Dialog.
		PostMessage(m_downloadThreadParam.hwndDlg, WM_USER_FINISHED, 0, 0);

		// Update the Tree Dialog.
		HWND hWnd = ((CApp *)AfxGetApp())->m_hwndTreeDlg;
		if(hWnd)
			PostMessage(hWnd, WM_USER_TREE_VID_UPDATE, 0, 0);

		// Clean up thread vars.
		m_downloadThreadParam.Zero();
	}
}

// Called from CDownloadDialog::OnButtonDownload() only.
// Note storage path has been validated by caller. 
BOOL CDoc::DownloadVideos(HWND hDlgWnd)
{
	// Ensure not in use.
	if(m_hDownloadThread != NULL)
		return FALSE;

	// Prepare.
	m_downloadEvent.ResetEvent();	// set nonsignaled

	// To allow the user to continue to add videos to the download
	// list while downloading the videos already on the list, we 
	// send the worker thread a copy of the download list.
	m_vidDnLdArTemp.Copy(m_vidDnLdAr);

	// Set param member.
	CMainFrame * pFrame = (CMainFrame*)(AfxGetMainWnd());
	if(!pFrame)
		return FALSE;
	m_downloadThreadParam.hwndMain	= pFrame->GetSafeHwnd();// CMainFrame HWND
	m_downloadThreadParam.hwndDlg	= hDlgWnd;				// CDownloadDialog HWND
	m_downloadThreadParam.pEvent	= &m_downloadEvent;		// CEvent
	m_downloadThreadParam.videoAr	= &m_vidDnLdAr;			// list of videos
	m_downloadThreadParam.path		= &m_videoStorePath;	// storage folder

	// Start worker thread in suspended state.
	CWinThread * pThread = AfxBeginThread(VideoDownloadThread, 
										  &m_downloadThreadParam,
										  THREAD_PRIORITY_NORMAL, 
										  0, 
										  CREATE_SUSPENDED);
	// Copy handle to m_hDownloadThread.
	::DuplicateHandle(GetCurrentProcess(),
					  pThread->m_hThread, 
					  GetCurrentProcess(), 
					  &m_hDownloadThread,
					  0, 
					  FALSE, 
					  DUPLICATE_SAME_ACCESS);
	// Start thread.
	pThread->ResumeThread();

	// OK
	return TRUE;
}

// Global Video Download Thread.
//
// Downloads videos in CDoc::m_vidDnLdAr array.
UINT VideoDownloadThread(LPVOID pParam)
{
	// Copy param data.
	THREADPARAM param = *(THREADPARAM *) pParam;

	// Message param.
	CString errMsg;

	// Call KhanInet.
	CKnanInet inet(param); 
	if(!inet.GetVideoFiles(errMsg))
	{
		// Inform user.
		//??

		// CKhanInet error.
		// Post thread abort message to CDoc via CMainFrame.
		PostMessage(param.hwndMain, WM_USER_DOWNLOAD_ABORTED, 0, 0);
		return 1;
	}

	// Download successfull - Finish.
	TRACE(_T("VideoDownloadThread successfull\n"));
	CString s;

	// Get message string for appending.
	CString msgStr;
	CApp * pApp = ((CApp*)AfxGetApp());
	pApp->GetThreadMessage(msgStr);
	msgStr += _T("\r\n");

	// Done message
	msgStr += _T("Video download Complete!\r\n");
	pApp->SetThreadMessage(msgStr);
	PostMessage(param.hwndDlg, WM_USER_STATUS_STRING, 0, 0);

	// Post thread finished msg to CDoc via CMainFrame.
	PostMessage(param.hwndMain, WM_USER_DOWNLOAD_FINISHED, 0, 0);

	TRACE(_T("VideoDownloadThread - processing complete\n"));

	return 0;
}

/*
// out 5/15 - fastly fixed
// in 5/20 - will test timed inet connection, and timed DirectShow Load().

////////////////////////////////////////////////////////////////////////
// Inet Stream Test Thread and video stream start up Handlers. 12/13/14

// see CDataView::StartVideoPlayback()

// Cancel Connection Test.
//  Called from View...
void CDoc::StreamTestCancel()
{
	// Signal the worker thread to terminate.
	//
	// Termination will not happen till OpenURL() 
	// in CKnanInet::TestInetStream2() returns or 
	// timesout.
	// Thread will call CDoc::InetStreamTestAborted() 
	// via CMainFrame.
    m_streamTestEvent.SetEvent();
}

// Called from CMainFrame::OnStreamTestFinished() - WM_USER_STREAM_TEST_FINISHED
// Indicates a valid stream is available.
void CDoc::StreamTestFinished()
{
	// Shut down the Stream Test worker thread.
	if(m_hStreamTestThread)
	{
		::WaitForSingleObject(m_hStreamTestThread, INFINITE);
		::CloseHandle (m_hStreamTestThread);
		m_hStreamTestThread = NULL;

		// Clean up thread vars.
		m_streamTestThreadParam.Zero();

		// Close the View Progress control
		CDataView * pView = GetViewPtr();
		if(pView)
			pView->StopResponseProgress();

		// Start LAV Video Stream.
		if(pView)
			pView->PlayVideoStream(m_curVideo.mp4);
	}
}

// Called from CMainFrame::OnStreamTestAborted() - WM_USER_DOWNLOAD_ABORTED
// Stream Test has been aborted by user or has failed.
void CDoc::StreamTestAborted()
{
	// Shut down the Stream Test worker thread.
	if(m_hStreamTestThread)
	{
		::WaitForSingleObject(m_hStreamTestThread, INFINITE);
		::CloseHandle (m_hStreamTestThread);
		m_hStreamTestThread = NULL;

		// Clean up thread vars.
		m_streamTestThreadParam.Zero();

		// Get View ptr.
		CDataView * pView = GetViewPtr();

		// Get err data from CApp.
		CString errMsg;
		DWORD errCode;
		int retCode;
		((CApp*)AfxGetApp())->GetThreadMessage(errMsg, errCode, retCode);

		// Close the View Progress control as needed.
		// If user has canceled the test, it will already be 
		// closed. If a timeout or error has occured, it will 
		// be running.
		if(pView)
			pView->StopResponseProgress();

		// If user canceled, or user canceled and a timeout occured.
		if(retCode == -2 || retCode == -3)
		{
			// Do nothing. 

			// dbg
			//AfxMessageBox(_T("cancelled returned"));
		}

		// Else a misc error (errCode == 0) or
		// a timeout (errCode == -1)/
		else
		{
			if(pView)
			{
				CString msg;
				if(retCode == -1 && errCode == 12002)
				{
					msg.Format(_T("%s\nYou can try again.\nError Code: %d"), errMsg, errCode);
					pView->MessageBox(msg, _T("Khan Desktop Application"), 
										MB_OK | MB_ICONINFORMATION);
				}
				else
				{
					msg.Format(_T("%s\nError Code: %d"), errMsg, errCode);
					pView->MessageBox(msg, _T("Khan Desktop Application"), 
										MB_OK | MB_ICONINFORMATION);
				}
			}
		}
		//
		// Do not start the LAV Stream.
		//
	}
}

// Called from CDataView::StartVideoPlayback() only.
// Note m_curVideo.mp4 must be a valid url. 
BOOL CDoc::StreamCurrentVideo()
{
	// Ensure thread has compleated normally.
	//
	// If the user cancels the operation, the thread will continue
	// to run until a connection is made or an error occurs, and
	// video playback is never started.
	//
	// StreamTestAborted() or StreamTestFinished() will eventually 
	// be called.
	//
	// So if the cancelled operation has not yet terminated 
	// normally, info the user she will have to wait. 
	// A better solution would be nice.
	if(m_hStreamTestThread != NULL)
	{
		CDataView * pView = GetViewPtr();
		if(pView)
			pView->MessageBox(_T("Reseting connection - please wait a few seconds."), 
							  _T("Khan Desktop Application"), 
									MB_OK | MB_ICONINFORMATION);
	
		return FALSE;
	}

	// Ensure a url.
	if(m_curVideo.mp4.IsEmpty())
		return FALSE;

	// Prepare.
	m_streamTestEvent.ResetEvent();	// set nonsignaled

	// Set param member.
	CMainFrame * pFrame = (CMainFrame*)(AfxGetMainWnd());
	if(!pFrame)
		return FALSE;
	m_streamTestThreadParam.hwndMain	= pFrame->GetSafeHwnd();// CMainFrame HWND
	m_streamTestThreadParam.hwndDlg		= NULL;					// Not used
	m_streamTestThreadParam.pEvent		= &m_streamTestEvent;	// CEvent
	m_streamTestThreadParam.path		= &m_curVideo.mp4;		// url

	// Start worker thread in suspended state.
	CWinThread * pThread = AfxBeginThread(TestInetStreamThread, 
										  &m_streamTestThreadParam,
										  THREAD_PRIORITY_NORMAL, 
										  0, 
										  CREATE_SUSPENDED);
	// Copy handle to m_hDownloadThread.
	::DuplicateHandle(GetCurrentProcess(),
					  pThread->m_hThread, 
					  GetCurrentProcess(), 
					  &m_hStreamTestThread,
					  0, 
					  FALSE, 
					  DUPLICATE_SAME_ACCESS);
	// Start thread.
	pThread->ResumeThread();

	// OK
	return TRUE;
}

// Global TestInetStreamThread for testing
// an Inet Stream connection.
UINT TestInetStreamThread(LPVOID pParam)
{
	// Copy param data.
	THREADPARAM param = *(THREADPARAM *) pParam;

	// Message param.
	CString errMsg;
	DWORD errCode;

	// Call KhanInet:TestInetStream2().
	// Will return:
	//   1 for success
	//	 0 for misc error
	//  -1 for timeout
	//	-2 for user cancel
	//	-3 for user canceled and exception
	//	-4 corrupt or missing video file
	//
	CKnanInet inet(param); 
	int ret = inet.TestInetStream(errMsg, errCode);

	// Success
	if(ret == 1)
	{
		TRACE(_T("TestInetStreamThread successfull\n"));

		// Post thread finished msg to CDoc::StreamTestFinished() 
		// via CMainFrame.
		PostMessage(param.hwndMain, WM_USER_STREAM_TEST_FINISHED, 0, 0);

		return 1;
	}

	// Failed
	else
	{
		TRACE(_T("TestInetStreamThread failed\n"));

		// Save err data to CApp for use by CDoc::StreamTestAborted().
		((CApp*)AfxGetApp())->SetThreadMessage(errMsg, errCode, ret);

		// Post thread aborted msg to CDoc::StreamTestAborted() 
		// via CMainFrame.
		PostMessage(param.hwndMain, WM_USER_STREAM_TEST_ABORTED, 0, 0);
	}

	return 0;
}
*/

//////////////////////////////////////////////////////////////
// Application update

// Called from CAppUpdateDlg::OnButtonUpdate().
// First to check if an update is available (checkForUpdate = TRUE).
// And again to do the update.
BOOL CDoc::UpdateApplication(HWND hDlgWnd, BOOL checkForUpdate)
{
	// Ensure not in use.
	if(m_hAppUpdateThread != NULL)
		return FALSE;

	// If checking for update, run the worker thread.
	// Thread will download valid update file (exe)
	// to app data folder.
	if(checkForUpdate)
	{
		// Prepare.
		m_appUpdateEvent.ResetEvent();		// set canceling to nonsignaled 

		// GetAppDataFolder() returns:
		//	vs6.0	- process folder.
		//	vs2005	- app data folder
		m_tempUpdatePath = ((CApp *)AfxGetApp())->GetAppDataFolder();
		m_tempUpdatePath += _T("\\");
		m_tempUpdatePath += NEW_EXE_STR;	// Khan_new.exe

		// Get current exe version.
		if(!GetVersion(m_curVersion))
			return FALSE;

		// Prepare param for KhanInet.
		CMainFrame * pFrame = (CMainFrame*)(AfxGetMainWnd());
		if(!pFrame)
			return FALSE;
		m_appUpdateThreadParam.hwndMain		= pFrame->GetSafeHwnd();// CMainFrame HWND
		m_appUpdateThreadParam.hwndDlg		= hDlgWnd;
		m_appUpdateThreadParam.pEvent		= &m_appUpdateEvent;	// CEvent
		m_appUpdateThreadParam.path			= &m_tempUpdatePath;
		m_appUpdateThreadParam.version		= &m_curVersion;

		// Start worker thread in suspended state.
		CWinThread * pThread = AfxBeginThread(AppUpdateThread, 
											  &m_appUpdateThreadParam,
											  THREAD_PRIORITY_NORMAL, 
											  0, 
											  CREATE_SUSPENDED);
		// Copy handle to m_hInetThread member.
		::DuplicateHandle(GetCurrentProcess(),
						  pThread->m_hThread, 
						  GetCurrentProcess(), 
						  &m_hAppUpdateThread,
						  0, 
						  FALSE, 
						  DUPLICATE_SAME_ACCESS);
		// Start thread.
		pThread->ResumeThread();

		return TRUE;
	}

	// Else user wants the Update installed.
	// Here we launch a separate process, KDPatch.exe, via ShellExecuteEx().
	// This should allow normal users to log on as an administrator - which
	// is required to write to the programs folder - and do the update.
	// This should be complient with Vista and Win7's UAC. 
	else
	{
		BOOL isError = FALSE;
		CString retStr;
		CString param;
		DWORD dwRet;

		// Get CApp.
		CApp * pApp = (CApp *)AfxGetApp();

		// Get current status message.
		CString dlgMsg;
		pApp->GetThreadMessage(dlgMsg);

		// Current directory.
		CString curDir = pApp->m_curDirectory + _T("\\");

		// Build parameters.
		CString appDataPath = m_tempUpdatePath;				// Khan_new.exe
		CString exePath		= curDir + CUR_EXE_STR;			// Khan.exe
		CString exeOldPath	= curDir + OLD_EXE_STR;			// Khan.exe.old

		// Format param string. Note paths may contain spaces, and so must
		// be enclosed in double quotes - spaces separate params.
		param.Format(_T("\"%s\" \"%s\" \"%s\""), exePath, exeOldPath, appDataPath);

		// Khan Desktop Patch.exe path.
		CString KDPatchPath =  curDir + KDPATCH_STR;		// KDPatch.exe

		// Prepare.
		SHELLEXECUTEINFO sei;								// unicode
		ZeroMemory(&sei, sizeof(SHELLEXECUTEINFOW));
		sei.cbSize			= sizeof(SHELLEXECUTEINFOW);
		sei.fMask			= SEE_MASK_NOCLOSEPROCESS ;		// get process handle in hProcess
		sei.hwnd			= NULL;
		sei.lpVerb			= _T("runas");
		sei.lpFile			= KDPatchPath;
		sei.lpParameters	= param;
		sei.lpDirectory		= NULL;
		sei.nShow			= SW_HIDE;						// SW_HIDE SW_SHOWNORMAL
		sei.hInstApp		= NULL;

		// This will launch the win RunAs dialog.
        if(ShellExecuteEx(&sei))							// unicode
		{
			// If KDPatch.exe launched.
			if(sei.hProcess)
			{
				// Wait until the process terminates.
				WaitForSingleObject(sei.hProcess, INFINITE); 

				// Get the exit code.				
				GetExitCodeProcess(sei.hProcess, &dwRet);

				// Success
				if(dwRet == 0)
				{
					// Update status msg.
					dlgMsg += _T("\r\n\r\nUpdate Successfull!\r\n")
							  _T("\r\nYou must restart Khan Desktop to\r\n")
						      _T("activate the Update.\r\n");
					pApp->SetThreadMessage(dlgMsg);
					TRACE(_T("CDoc UpdateApplication Update successfull.\n"));
					PostMessage(hDlgWnd, WM_USER_STATUS_STRING, 0, 0);

					// Only update once per app instance.
					m_appUpdateInstalled = TRUE;

					// Ok.
					return TRUE;
				}

				// KDPatch.exe error.
				else
				{
					// Set error message.
					retStr.Format(_T("\r\n\r\nUpdate failed with code: %d"), dwRet);
					//
					// MFC not initialized.
					if(dwRet == 1)
						retStr += _T("\r\nKDPatch: Proccess not initialized.");
					// Not an administrator.
					else if(dwRet == 2)
						retStr += _T("\r\nKDPatch: Administrator required to install the update.");
					// Invalid number of parameters.
					else if(dwRet == 3)
						retStr += _T("\r\nKDPatch: Invalid number of parameters.");
					// Invalid update exe path.
					else if(dwRet == 4)
						retStr += _T("\r\nKDPatch: Invalid update exe path.");
					// Invalid program exe path.
					else if(dwRet == 5)
						retStr += _T("\r\nKDPatch: Invalid program exe path.");
					// Shell Execute error.
					else if(dwRet == 7)
						retStr += _T("\r\nKDPatch: ShellExecute error.");
					
					isError = TRUE;
				}
			}
			// hProcess will be NULL if no process launched.
			else
			{
				// Set err message.
				retStr.Format(_T("\r\nKDPatch failed to run the Update."));
				isError = TRUE;
			}
		}
		// Error with ShellExecute.
		else
		{
			dwRet = GetLastError();
			switch(dwRet)
			{
				// Pressed RunAs dialog's cancel.
				case ERROR_CANCELLED:	
				{
					// Info user.
					retStr = _T("\r\nUpdate cancelled.");
					isError = TRUE;
				}
				break;
				case ERROR_ACCESS_DENIED:
				{
					retStr.Format(_T("\r\nUpdate failed: access denied - code %d"), dwRet);
					isError = TRUE;
				}
				break;
				case ERROR_FILE_NOT_FOUND:
				case ERROR_PATH_NOT_FOUND:
				case ERROR_DDE_FAIL:
				case ERROR_NO_ASSOCIATION:
				case ERROR_DLL_NOT_FOUND:
				case ERROR_NOT_ENOUGH_MEMORY:
				case ERROR_SHARING_VIOLATION:
				default:
				{
					retStr.Format(_T("\r\nUpdate failed - ShellExecuteEx error: %d"), dwRet);
					isError = TRUE;
				}
				break;
			}
		}
		// Handle error.
		if(isError)
		{
			// Delete downloaded file if exists.
	//		CleanCanceledAppUpdate();

			// Info user.
			dlgMsg += retStr;
			pApp->SetThreadMessage(dlgMsg);
			PostMessage(hDlgWnd, WM_USER_STATUS_STRING, 0, 0);
			retStr.TrimLeft();
			retStr.TrimRight();
			CString s;
			s.Format(_T("CDoc UpdateApplication failed:\n%s\n"), retStr);
			TRACE(s);

			// Update dialog of error - lParam == 0.
			// Note dialog will call pDoc::CleanCanceledAppUpdate();
			PostMessage(hDlgWnd, WM_USER_FINISHED, 0, 0);
		}

		return FALSE;
	}
	// Should not get here.
	return FALSE;
}

// Called from CAppUpdateDlg::OnButtonUpdate().
// to cancel the Application Update.
void CDoc::CancelAppUpdate()
{
	// Kill the m_hAppUpdateThread worker thread.
	//
	// Worker Thread will call CDoc::AppUpdateAborted() 
	// via CMainFrame.
    m_appUpdateEvent.SetEvent();
}

// Remove downloaded left over temp file.
void CDoc::CleanCanceledAppUpdate()
{
	CString downloadPath = ((CApp *)AfxGetApp())->GetAppDataFolder();
	downloadPath += _T("\\");
	downloadPath += NEW_EXE_STR;

	// If the temp download exists, delete it.
	CFileStatus status;
	if(CFile::GetStatus(downloadPath, status)) // static version for test.
	{
		TRACE(_T("CDoc CleanCanceledAppUpdate removing temp file.\n"));
		CFile::Remove(downloadPath);
	}
}

// Called directly from CMainFrame::OnAppUpdateFinished().
void CDoc::AppUpdateFinished()
{
	// Shut down the App Update worker thread.
	if(m_hAppUpdateThread)
	{
		::WaitForSingleObject(m_hAppUpdateThread, INFINITE);
		::CloseHandle (m_hAppUpdateThread);
		m_hAppUpdateThread = NULL;

		// New exe has been stored temporarilly in app data forlder.
		// Inform dialog - lParam == 1.
		PostMessage(m_appUpdateThreadParam.hwndDlg, WM_USER_FINISHED, 0, 1);

		// Clean up thread vars.
		m_appUpdateThreadParam.Zero();
	}
}

// Called from AppUpdateThread() via user message to
// CMainFrame::OnAppUpdateAborted() - WM_USER_APP_UPDATE_ABORTED.
void CDoc::AppUpdateAborted()
{
	// Shut down the App Update worker thread.
	if(m_hAppUpdateThread)
	{
		::WaitForSingleObject(m_hAppUpdateThread, INFINITE);
		::CloseHandle (m_hAppUpdateThread);
		m_hAppUpdateThread = NULL;

		// Update the AppUpdate Dialog - lParam == 0.
		PostMessage(m_appUpdateThreadParam.hwndDlg, WM_USER_FINISHED, 0, 0);

		// Clean up thread vars.
		m_appUpdateThreadParam.Zero();
	}
}

// Global Application Update Thread
//
// Update app with SourceForge.net data.
//
// Although these are tiny downloads, inet delays can
// occure, freezing up everyting if not threaded. So
// make it bomb proof with a worker thread.
UINT AppUpdateThread(LPVOID pParam)
{
	// Copy param data.
	THREADPARAM param = *(THREADPARAM *) pParam;

	// Call KhanInet.
	CKnanInet inet(param);

	// Returns 1 for update downloaded and ready to be installed.
	// Returns 0 for no update needed.
	// Returns -1 for error.
	int ret = inet.GetAppUpdate();

	if(ret == 1)
	{
		TRACE(_T("AppUpdateThread - update ready to be installed.\n"));

		// Post thread finished msg to CDoc via CMainFrame.
		PostMessage(param.hwndMain, WM_USER_APP_UPDATE_FINISHED, 0, 0);
		return 0;
	}
	// Else update not needed or an error.
	else
	{
		// CKhanInet error.
		// Post thread abort message to CDoc via CMainFrame.
		PostMessage(param.hwndMain, WM_USER_APP_UPDATE_ABORTED, 0, 0);
		return 1;
	}
}

//////////////////////////////////////////////////////////////
// DirectShow player.

// Get the m_curVideo path.
BOOL CDoc::GetCurrentVideoPath(CString & path, BOOL & isUrl)
{
	if(m_curVideo.mp4.IsEmpty())
		return FALSE;

	// Check if on disk.
	if(VideoOnDisk(m_curVideo))
	{
		isUrl = FALSE;
		CString fileName;
		if(m_curVideo.GetShortFileName(fileName))
		{
			path = m_videoStorePath;
			path += _T('\\');
			path += fileName;
			return TRUE;
		}
	}
	else
	{
		isUrl = TRUE;
		path = m_curVideo.mp4;
		return TRUE;
	}
	return FALSE;
}

//////////////////////////////////////////////////////////////
// Display String handling

// Video data.
// Called from CTreeDlg.
void CDoc::SetDisplayStringsVideo(BOOL updateView)
{
	CString str, str2;

	// Vid duration.
	int time = _wtoi(m_curVideo.duration);
	int minutes = time / 60;
	int seconds = time % 60;

	// History.
	double timesViewed = 0;
	CTime lastViewed = CTime::GetCurrentTime();
	if(m_history.GetHistoryDisplayData(m_curVideo, timesViewed, lastViewed))
	{
		int hour = lastViewed.GetHour();
		int min  = lastViewed.GetMinute();
		CString am  = hour < 12 ? _T("AM") : _T("PM");
		if(hour > 12)
			hour = hour - 12;

		str2.Format(_T("Times Viewed:\n")
					_T("\t%.2f\n\n")
					_T("Last Viewed:\n")
					_T("\t%d/%d/%d  %d:%.2d %s\n\n"),
							timesViewed,
							lastViewed.GetMonth(),
							lastViewed.GetDay(),
							lastViewed.GetYear(), 
							hour,
							min,
							am );						
	}
	else
	{
		str2 = _T("\n\n");
	}

	// Format.
	str.Format(	_T("Video Title:\n")
				_T("\t%s\n\n")
				_T("Description:\n")
				_T("\t%s\n\n")
				_T("Duration:\n")
				_T("\t%d Min. %d Sec.\n\n")
				_T("%s"),
						m_curVideo.title,
						m_curVideo.description,
						minutes, seconds,
						str2);
	// Set member.
	m_rawStr = str;

	// Redraw view if no vid playing.
	if(!m_videoPlaying && updateView)
		UpdateAllViews(NULL, RAW_STRING, NULL);

	// If ListFiles is showing, set its selection to the video.
	if(m_listFiles.ListShowing())
	{
		m_listFiles.SetCurSelVideo();
	}
}

// Topic data.
// Called from CTreeDlg.
void CDoc::SetDisplayStringsTopic(BOOL updateView)
{
	CString str, str2;
	CString title = m_curTopic.title;
	CString description = m_curTopic.description;
	if(description.IsEmpty())
		description = _T("None");

	// If Topic has Child Topics.
	if(m_nChildTopics)
	{
		str.Format(	_T("Topic Title:\n")
					_T("\t%s\n\n")
					_T("Description:\n")
					_T("\t%s"), 
						title, description);

		// If Videos along with topics - check "linear algebra" topic.
		if(m_nVideos)
		{
			int minutes = m_nTotalVideoTime / 60; // fix v0.9.4
			int hours   = minutes / 60;
			if(minutes >= 60)
				minutes = minutes % 60;
			CString time;
			if(hours)
				time.Format(_T("%d Hours and %d Minutes"), hours, minutes);
			else
				time.Format(_T("%d Minutes"), minutes);
			int nViewed = m_history.TopicVideosViewed(m_curTopic.treeTopic);

			if(nViewed)
			{
				str2.Format(_T("\n\nContains:\n")
							_T("\t%d Topics and %d Videos\n\n")
							_T("Total Video Time:\n")
							_T("\t%s\n\n") 
							_T("Viewed:\n")
							_T("\t%d Videos\n"), 
								m_nChildTopics, m_nVideos,
								time,
								nViewed);
			}
			else
			{
				str2.Format(_T("\n\nContains:\n")
							_T("\t%d Topics and %d Videos\n\n")
							_T("Total Video Time:\n")
							_T("\t%s\n"), 
								m_nChildTopics, m_nVideos,
								time);
			}
			str += str2;
		}
		// Else only Child Topics,
		else
		{
			str2.Format(_T("\n\nContains:\n")
						_T("\t%d Topics"), 
							m_nChildTopics);
			str += str2;
		}
	}

	// Else if Topic has Videos.
	else if(m_nVideos)
	{
		int minutes = m_nTotalVideoTime / 60; // fix v0.9.4
		int hours   = minutes / 60;
		if(minutes >= 60)
			minutes = minutes % 60;
		CString time;
		if(hours)
			time.Format(_T("%d Hours and %d Minutes"), hours, minutes);
		else
			time.Format(_T("%d Minutes"), minutes);
		int nViewed = m_history.TopicVideosViewed(m_curTopic.treeTopic);

		if(nViewed)
		{
			str.Format(	_T("Topic Title:\n")
						_T("\t%s\n\n")
						_T("Description:\n")
						_T("\t%s\n\n")
						_T("Contains:\n")
						_T("\t%d Videos\n\n")
						_T("Total Video Time:\n")
						_T("\t%s\n\n")
						_T("Viewed:\n")
						_T("\t%d Videos\n"), 
							title, description, 
							m_nVideos,
							time,
							nViewed);
		}
		else
		{
			str.Format(	_T("Topic Title:\n")
						_T("\t%s\n\n")
						_T("Description:\n")
						_T("\t%s\n\n")
						_T("Contains:\n")
						_T("\t%d Videos\n\n")
						_T("Total Video Time:\n")
						_T("\t%s\n"), 
							title, description, 
							m_nVideos,
							time);
		}
	}

	// Set member.
	m_rawStr = str;

	// Redraw view if no vid playing.
	if(!m_videoPlaying && updateView)
		UpdateAllViews(NULL, RAW_STRING, NULL);
}

// Set the location for Video storage.
void CDoc::OnSetupVideoStore() 
{
	CVideoStoreDlg dlg;
	if(dlg.DoModal() == IDOK)
	{
		// Set CDoc member.
		m_videoStorePath = dlg.m_strFolderPath;
	}	
}
// Update.
void CDoc::OnUpdateSetupVideoStore(CCmdUI* pCmdUI) 
{
	// No vid playback or updating.
	if(m_videoPlaying			|| 
	   m_hUpdateThread			||
	   m_hDownloadThread		||
	   m_hAppUpdateThread		||
//	   m_hStreamTestThread		|| // 12/13/14
	   m_listFiles.ListShowing() )
		pCmdUI->Enable(FALSE);
	else
		pCmdUI->Enable(TRUE);	
}

void CDoc::OnApplicationUpdate() 
{
	// TODO: Add your command handler code here
	CAppUpdateDlg dlg;
	if(dlg.DoModal() == IDOK)
	{
		// If no update was installed, clean any
		// left over temp file.
		if(!m_appUpdateInstalled)
			CleanCanceledAppUpdate();
	}		
}

// GetSecurityInfo

// Check if the caller belongs to the Administrators group.
BOOL CDoc::IsAdmin()
{

	// See also CheckTokenMembership CheckTokenMembershipEx IsUserAnAdmin();

#if VS2005
	// From: msdn CheckTokenMembership @
	// http://msdn.microsoft.com/en-us/library/windows/desktop/aa376389%28v=vs.85%29.aspx
	// Minimum supported client Windows XP [desktop apps only]
	//
	// Note this same code in run in KDPatch.exe for application updates.
	// Does work on 03 Server.
	// On Win7, will return false for an admin due to the the split token,
	// but after being prompted by UAC to elevate, returns true.
	// So, for App Update, KDPatch.exe is manifested for an admin, which
	// will trigger the UAC elevation, and allow KDPatch.exe ro run.
	SID_IDENTIFIER_AUTHORITY NtAuthority = SECURITY_NT_AUTHORITY;
	PSID AdministratorsGroup; 
	BOOL b = AllocateAndInitializeSid(&NtAuthority,
									  2,
									  SECURITY_BUILTIN_DOMAIN_RID,
									  DOMAIN_ALIAS_RID_ADMINS,
									  0, 0, 0, 0, 0, 0,
									  &AdministratorsGroup); 
	if(b) 
	{
		if(!CheckTokenMembership( NULL, AdministratorsGroup, &b)) 
		{
			b = FALSE;
		} 
		FreeSid(AdministratorsGroup); 
	}

	return b;

#else 

	// VC6.0
	// From: http://support.microsoft.com/kb/171273
	// NT 4.0 and up
	//
	// Make up some private access rights.
	#define ACCESS_READ  1
	#define ACCESS_WRITE 2


	HANDLE hToken;
	DWORD  dwStatus;
	DWORD  dwAccessMask;
	DWORD  dwAccessDesired;
	DWORD  dwACLSize;
	DWORD  dwStructureSize = sizeof(PRIVILEGE_SET);
	PACL   pACL            = NULL;
	PSID   psidAdmin       = NULL;
	BOOL   bReturn         = FALSE;

	PRIVILEGE_SET   ps;
	GENERIC_MAPPING GenericMapping;

	PSECURITY_DESCRIPTOR     psdAdmin           = NULL;
	SID_IDENTIFIER_AUTHORITY SystemSidAuthority = SECURITY_NT_AUTHORITY;
   
	__try 
	{

		// AccessCheck() requires an impersonation token.
		ImpersonateSelf(SecurityImpersonation);

		if (!OpenThreadToken(GetCurrentThread(), TOKEN_QUERY, FALSE, &hToken)) 
		{

			if (GetLastError() != ERROR_NO_TOKEN)
				__leave;

			// If the thread does not have an access token, we'll 
			// examine the access token associated with the process.
			if (!OpenProcessToken(GetCurrentProcess(), TOKEN_QUERY, &hToken))
				__leave;
		}

		if(!AllocateAndInitializeSid(&SystemSidAuthority, 2, 
										SECURITY_BUILTIN_DOMAIN_RID, 
										DOMAIN_ALIAS_RID_ADMINS,
										0, 0, 0, 0, 0, 0, &psidAdmin) )
			__leave;

		psdAdmin = LocalAlloc(LPTR, SECURITY_DESCRIPTOR_MIN_LENGTH);
		if(psdAdmin == NULL)
			__leave;

		if(!InitializeSecurityDescriptor(psdAdmin, 
										 SECURITY_DESCRIPTOR_REVISION))
			__leave;

		// Compute size needed for the ACL.
		dwACLSize = sizeof(ACL)				 + 
		sizeof(ACCESS_ALLOWED_ACE) +
				GetLengthSid(psidAdmin)	 - 
				sizeof(DWORD);

		// Allocate memory for ACL.
		pACL = (PACL)LocalAlloc(LPTR, dwACLSize);
		if(pACL == NULL)
			__leave;

		// Initialize the new ACL.
		if(!InitializeAcl(pACL, dwACLSize, ACL_REVISION2))
			__leave;

		dwAccessMask = ACCESS_READ | ACCESS_WRITE;

		// Add the access-allowed ACE to the DACL.
		if(!AddAccessAllowedAce(pACL, ACL_REVISION2,
								dwAccessMask, psidAdmin))
			__leave;

		// Set our DACL to the SD.
		if(!SetSecurityDescriptorDacl(psdAdmin, TRUE, pACL, FALSE))
			__leave;

		// AccessCheck is sensitive about what is in the SD; set
		// the group and owner.
		SetSecurityDescriptorGroup(psdAdmin, psidAdmin, FALSE);
		SetSecurityDescriptorOwner(psdAdmin, psidAdmin, FALSE);

		if(!IsValidSecurityDescriptor(psdAdmin))
			__leave;

		dwAccessDesired = ACCESS_READ;

		// 
		// Initialize GenericMapping structure even though we
		// won't be using generic rights.
		// 
		GenericMapping.GenericRead    = ACCESS_READ;
		GenericMapping.GenericWrite   = ACCESS_WRITE;
		GenericMapping.GenericExecute = 0;
		GenericMapping.GenericAll     = ACCESS_READ | ACCESS_WRITE;

		if (!AccessCheck(psdAdmin, hToken, dwAccessDesired, 
						 &GenericMapping, &ps, &dwStructureSize, &dwStatus, 
						 &bReturn)) 
		{
			TRACE(_T("AccessCheck() failed with error %lu\n"), GetLastError());
			__leave;
		}

		RevertToSelf();

	}

	__finally 
	{
		// Cleanup 
		if(pACL) LocalFree(pACL);
		if(psdAdmin) LocalFree(psdAdmin);  
		if(psidAdmin) FreeSid(psidAdmin);
	}

	return bReturn;

#endif
}

/////////////////////////////////////////////////////////////////
// Tools

// Delete unused video files from storage folder.
void CDoc::OnToolsOrphanVideos() 
{
	// CDeleteOrphans does all.
	CDeleteOrphans dlg;
	if(dlg.DoModal() == IDOK)
	{
		int g = 0;
	}	
}
// Update.
void CDoc::OnUpdateToolsOrphanVideos(CCmdUI* pCmdUI) 
{
	// No vid playback or updating.
	if(m_videoPlaying			|| 
	   m_hUpdateThread			||
	   m_hDownloadThread		||
	   m_hAppUpdateThread		||
//	   m_hStreamTestThread		|| // 12/13/14
	   m_listFiles.ListShowing() )
		pCmdUI->Enable(FALSE);
	else
		pCmdUI->Enable(TRUE);	
}

// Toggle the ListFiles display via CView.
// Called from Tools Menu and CListFiles::OnContextMenu().
void CDoc::OnToolsListFiles() 
{
	// CDataview handles all.
	// See CDataView::ToggleFilesList().
	UpdateAllViews(NULL, TOGGLE_LIST_FILES, NULL);

}
// Update
void CDoc::OnUpdateToolsListFiles(CCmdUI* pCmdUI) 
{
	// No vid playback or updating.
	if(m_videoPlaying	   || 
	   m_hUpdateThread	   ||
	   m_hDownloadThread   ||
//	   m_hStreamTestThread || // 12/13/14
	   m_hAppUpdateThread  )
		pCmdUI->Enable(FALSE);
	else
		pCmdUI->Enable(TRUE);	
}

/////////////////////////////////////////////////////////////////////
// User Data

// Read user data. 
// Data should be set to defaults first in case this fails.
BOOL CDoc::ReadUserData()
{
	int ret = TRUE;

	// Get User App Data
//	CString curDir = ((CApp*)AfxGetApp())->GetUserSettingsFolder();
//	curDir += _T("\\user.dat");
	CString curDir;
	if( ((CApp*)AfxGetApp())->GetUserSettingsFolder(curDir))
	{
		curDir += _T("\\user.dat");
	}
	else
		return FALSE;

	// Open the file for reading.
	CFile file;
	CFileException ex;
	if(!file.Open(curDir, CFile::modeRead | CFile::shareDenyWrite, &ex))
	{
		//ex.ReportError();
		return FALSE;
	}
	if(file.GetLength() == 0)
	{
		TRACE(_T("ERROR CDoc ReadUserData GetLength() = 0\n"));
		return FALSE;
	}

	CArchive ar(&file, CArchive::load);
	TRY
	{	
		// Get schema
		int schema; // note GetObjectSchema() is for serializable classes
		ar >> schema;
		if(schema != 1)
		{
			MessageBox(NULL,
					   _T("User data file could not be read."), 
					   _T("Khan Desktop Application"), 
					   MB_OK | MB_ICONINFORMATION);
			
			// Close Archive
			ar.Close();
			// Close the file and ignore any assertions	
			file.Abort();
			return FALSE;
		}

		// Read videos not yet downloaded.
		int sz;
		ar >> sz;
		CVideo video;
		for(int i = 0; i < sz; i++)
		{
			video.Read(ar);
			m_vidDnLdAr.Add(video);
			video.Zero();
		}

		// Read LOGFONTS.
		ar.Read(&m_viewLf, sizeof(LOGFONT));
		ar.Read(&m_treeLf, sizeof(LOGFONT));

		// Read video storage location.
		ar >> m_videoStorePath;

		// Read temp update file sz. Used for UpdateKhanLibrary() progess ctrl.
		ar >> m_tmpUpdateFileSz;

		// Close the archive.	
		ar.Close();
		// Close the file	
		file.Close();
	}
	CATCH_ALL(e)
	{
		// Close the archive.	
		ar.Close();
		// Close file safely and quietly
		file.Abort(); 
		TRY
		{
			// CDoc member
			//ReportSaveLoadException(filePath, e, FALSE, AFX_IDP_FAILED_TO_OPEN_DOC);
		}
		END_TRY
		e->Delete();
		ret = FALSE;
	}
	END_CATCH_ALL

	return ret;
}

// Write user data.
BOOL CDoc::WriteUserData()
{
	int ret = TRUE;

	// Get User App Data
//	CString curDir = ((CApp*)AfxGetApp())->GetUserSettingsFolder();
//	curDir += _T("\\user.dat");	
	CString curDir;
	if( ((CApp*)AfxGetApp())->GetUserSettingsFolder(curDir))
	{
		curDir += _T("\\user.dat");
	}
	else
		return FALSE;

	// Open the file for writing.
	CFile file;
	CFileException ex;
	if(!file.Open(curDir, CFile::modeCreate | CFile::modeWrite , &ex))
	{
		ex.ReportError();
		return FALSE;
	}

	CArchive ar(&file, CArchive::store);
	TRY
	{
		// Set schema
		int schema = 1; // note SetObjectSchema() is for serializable classes
		ar << 1;

		// Write video files still marked for download.
		int sz = m_vidDnLdAr.GetSize();
		ar << sz;
		for(int i = 0; i < sz; i++)
		{
			m_vidDnLdAr[i].Write(ar);
		}

		// Write LOGFONTS.
		ar.Write(&m_viewLf, sizeof(LOGFONT));
		ar.Write(&m_treeLf, sizeof(LOGFONT));

		// Write video storage location.
		ar << m_videoStorePath;

		// Write temp update file sz. Used for UpdateKhanLibrary() progess ctrl.
		ar << m_tmpUpdateFileSz;

		// Close the archive.	
		ar.Close();
		// Close the file	
		file.Close();
	}
	CATCH_ALL(e)
	{
		// Close the archive.	
		ar.Close();
		// Close file safely and quietly
		file.Abort(); 
		TRY
		{
			// CDoc member
			//ReportSaveLoadException(curDir, e, TRUE, AFX_IDP_FAILED_TO_SAVE_DOC);
			// THROW_LAST();
		}
		END_TRY
		e->Delete();
		ret = FALSE;
	}
	END_CATCH_ALL

	return ret;
}


/////////////////////////////////////////////////////////////////////
// Utilities

// Get application Version, e.g. 0.4.0.0, into strVersion.
BOOL CDoc::GetVersion(CString & strVersion)
{
	// From: MSDN Apr98CQAcode.exe - Paul DiLascia

	// Get module handle
	TCHAR filename[_MAX_PATH];
	HMODULE hModule = ::GetModuleHandle(NULL);
	if(!hModule)
		return FALSE;

	// Get module file name
	DWORD len = GetModuleFileName(hModule, filename,
								  sizeof(filename) / sizeof(filename[0]));
	if(len <= 0)
		return FALSE;

	// Read file version info. Handle will always be set to zero.
	DWORD dwDummyHandle;					
	len = GetFileVersionInfoSize(filename, &dwDummyHandle);
	if(len <= 0)
		return FALSE;

	// Allocate version info	
	BYTE * pVersionInfo = new BYTE[len];			
	if (!::GetFileVersionInfo(filename, 0, len, pVersionInfo))
	{
		if(pVersionInfo)
			delete [] pVersionInfo;
		return FALSE;
	}

	// Get VS_FIXEDFILEINFO.
	VS_FIXEDFILEINFO * fixedInfo = NULL;
	UINT iLen;
	if(!VerQueryValue(pVersionInfo, _T("\\"), (void **) &fixedInfo, &iLen))
	{
		if(pVersionInfo)
			delete [] pVersionInfo;
		return FALSE;
	}

	// Set version string with VS_FIXEDFILEINFO data.
	strVersion.Format(_T("%d.%d.%d.%d"), HIWORD(fixedInfo->dwFileVersionMS), 
										 LOWORD(fixedInfo->dwFileVersionMS),
										 HIWORD(fixedInfo->dwFileVersionLS), 
										 LOWORD(fixedInfo->dwFileVersionLS) ); 
	// Clean up.
	if(pVersionInfo)
		delete [] pVersionInfo;


	// dbg test
//	CString v2 = _T("0.6.0.0\n");
//	int ret = CompareVersions(strVersion, v2);

	return TRUE;
}

// Compare two version strings. For example: 0.4.0.0 and 0.5.0.0.
// Return  1 if v1 > v2. 
// Return  0 if v1 <= v2. 
// Return -1 an error.
int CDoc::CompareVersions(const CString & v1, const CString & v2)
{
	int sz1 = v1.GetLength();
	int sz2 = v2.GetLength();
	if(sz1 < 7 || sz2 < 7)		// newline ok at end.
		return -1;

	TCHAR ch1, ch2;
	for(int i = 0; i < 7; i++)
	{
		ch1 = v1[i];
		ch2 = v2[i];

		// Ensure a digit or a dot.
		if(ch1 == _T('.'))
			continue;
		else
		{
			if(!_istdigit(ch1) || !_istdigit(ch2))
				return -1;
		}

		// v1 is older.
		if(v1[i] < v2[i])
			return 0;
		// v1 is newer
		else if(v1[i] > v2[i])
			return 1;
	}

	// Equal.
	return 0;
}






