
/*	Copyright (C) G. Jespersen
	See Licence.txt.*/

// Dump.cpp : implementation file - v0.9.2.0
//
// Primary problem: The Parameter is Incorrect msg when selecting
// a video on win7:
//
// Note: Add dbghelp.lib to Project/Settings/Link/Object Library Modules
//
// Additional settings per: 
//		http://msdn.microsoft.com/en-us/library/ee416349%28v=vs.85%29.aspx#writing_a_minidump
//	Set Debug Information Format to Program Database (/Zi)
//	Set Generate Debug Info to Yes (/DEBUG)
//	Set References to Eliminate Unreferenced Data (/OPT:REF)
//	Enable COMDAT Folding to Remove Redundant COMDATs (/OPT:ICF) - no, see vlc below.
//	Enable C++ Exceptions EHa
//	Optimizations none C\C++ - must or some tests produced usless dumps
//	Inline Function Expansion - default
//
// Problems - VLC - vs2005 - vc6:
//	(/OPT:REF) and (/OPT:ICF) per above crash vlc in release.
//	also: Linker/Enable Incremental Linking must be YES.
//	see discussion at: 
//	 http://stackoverflow.com/questions/8885259/fixing-libvlc-release-mode-crash-with-vc2010
//
// Notes:
//	Testing causes DEP activation on vs2005 - /NXCOMPAT is for dep
//	 see: http://blogs.msdn.com/b/gauravb/archive/2008/09/23/disable-dep-on-applications.aspx
//		  http://blogs.msdn.com/b/vcblog/archive/2009/05/21/dynamicbase-and-nxcompat.aspx
//	_CRT_SECURE_NO_WARNINGS #pragma warning(disable: 4996)
//
// See also:
//	CApp::Run() for initialization.
//	CApp::ProcessWndProcException() for mfc exceptions.
//	CDataView::OnTest() for main thread test results.
//

#include "stdafx.h"
#include "Khan.h"
#include "Dump.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


////////////////////////////////////////////////////////////////////
// Global - hacked from: 
//	www.debuginfo.com, Author: Oleg Starodumov
//	www.randomascii.wordpress.com/2012/07/22/more-adventures-in-failing-to-crash-properly/
//
//
// Global Variables
CString gStrDumpPath;
CString gStrSuccessMsg;
BOOL gbDone = FALSE;


// Called at program startup only from CApp::Run().
void SetKhanExceptionFilter(const CString & dumpFilePath)
{
	// Set the filter.
	if(!SetUnhandledExceptionFilter(KhanExceptionFilter))
	{
		TRACE(_T("WARNING SetUnhandledExceptionFilter returned NULL"));
	}

	// Initialize global dump file path.
	gStrDumpPath = dumpFilePath;

	// Initialize global success message.
	gStrSuccessMsg.Format(_T("An error occured - Khan Desktop must close.\n")
						  _T("An error report has been created at:\n\n%s\n\n")
						  _T("To help Khan Desktop fix this problem, please ")
						  _T("see the Report Errors help topic."), gStrDumpPath);

	// Set up a handler for invalid parameters such as printf(NULL);
	#if VS2005
	_set_invalid_parameter_handler(HandleInvalidParameter);
	#endif

}

// Called by system.
LONG __stdcall KhanExceptionFilter(EXCEPTION_POINTERS * pep) 
{
	// Write dump only once. MiniDumpWriteDump() may cause 
	// another exception and overwrite the original file.
	if(gbDone)
		return EXCEPTION_EXECUTE_HANDLER; // 1
	gbDone = TRUE;

	// Normal dump - 18 kb .dmp file
	CreateMiniDump(pep); 

	return EXCEPTION_EXECUTE_HANDLER; 
}

// Mini Dump - Normal
void CreateMiniDump(EXCEPTION_POINTERS * pep) 
{
	// dbg - ok
	//if(!pep)
	//	AfxMessageBox(_T("No EXCEPTION_POINTERS"));

	// Open the file 
	HANDLE hFile = CreateFile(gStrDumpPath, 
							  GENERIC_READ | GENERIC_WRITE,
							  0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL ); 

	if(hFile != NULL && hFile != INVALID_HANDLE_VALUE) 
	{
		// Create the minidump
		MINIDUMP_EXCEPTION_INFORMATION mdei; 
		mdei.ThreadId           = GetCurrentThreadId(); 
		mdei.ExceptionPointers  = pep; 
		mdei.ClientPointers     = FALSE; 
		MINIDUMP_TYPE mdt       = MiniDumpNormal; 

		BOOL rv = MiniDumpWriteDump(GetCurrentProcess(), GetCurrentProcessId(), 
									hFile, mdt, (pep != 0) ? &mdei : 0, 0, 0 ); 
		if(!rv)
			MessageBox(NULL, _T("An error occured - Khan Desktop must close - dump not available."), _T("Khan Desktop Error"), MB_OK);
		else 
			MessageBox(NULL, gStrSuccessMsg, _T("Khan Desktop Error"), MB_OK);

		// Close the file.
		CloseHandle( hFile );
	}
	else 
	{
		MessageBox(NULL, _T("An error occured - Khan Desktop must close - dump file not created."), _T("Khan Desktop Error"), MB_OK);
	}
}


#if VS2005
// Invalid-parameter.
void __cdecl HandleInvalidParameter(const wchar_t *, const wchar_t *, 
									const wchar_t *, unsigned int, uintptr_t)
{
	// Note all params NULL in release.
	//
	__debugbreak();
	TerminateProcess(GetCurrentProcess(), 10);
}
#endif

// test load
// Note from msdn: Using Run-Time Dynamic Linking
//	Because the program uses run-time dynamic linking, you 
//	should not link with the import library when creating 
//	the program module.
bool __cdecl LoadDbgDll(void)
{
	return FALSE;

	// see: https://duilib.googlecode.com/svn/trunk/GameDemo/MiniDumper.cpp

	// AfxLoadLibrary

typedef BOOL
(WINAPI * PFNMINIDUMPWRITEDUMP)(
    IN HANDLE hProcess,
    IN DWORD ProcessId,
    IN HANDLE hFile,
    IN MINIDUMP_TYPE DumpType,
    IN CONST PMINIDUMP_EXCEPTION_INFORMATION ExceptionParam, OPTIONAL
    IN CONST PMINIDUMP_USER_STREAM_INFORMATION UserStreamParam, OPTIONAL
    IN CONST PMINIDUMP_CALLBACK_INFORMATION CallbackParam OPTIONAL   ) ;

	// Firstly see if dbghelp.dll is around and has the function we need
	// look next to the EXE first, as the one in System32 might be old 
	// (e.g. Windows 2000)
	HMODULE hDll = NULL;
	typedef UINT (CALLBACK* LPFNDLLFUNC1)(DWORD,UINT);
	LPFNDLLFUNC1 lpfnDllFunc1;    // Function pointer
	TCHAR szDbgHelpPath[_MAX_PATH];

	if (GetModuleFileName( NULL, szDbgHelpPath, _MAX_PATH ))
	{
		TCHAR *pSlash = _tcsrchr( szDbgHelpPath, _T('\\') );
		if (pSlash)
		{
			//_tcscpy( pSlash+1, _T("DBGHELP.DLL") );
			StringCbCopy(pSlash + 1, 12 * sizeof(TCHAR), _T("DBGHELP.DLL")); 

			hDll = ::LoadLibrary( szDbgHelpPath );
		}
	}

	if (hDll==NULL)
	{
		// load any version we can
		hDll = ::LoadLibrary( _T("DBGHELP.DLL") );
	}

	if(hDll)
	{
		lpfnDllFunc1 = (LPFNDLLFUNC1)GetProcAddress(hDll, "MiniDumpWriteDump");
		if(!lpfnDllFunc1)
			int g = 0;

		// Free
		BOOL ret = FreeLibrary(hDll);

	}

	LPCTSTR szResult = NULL;

	return FALSE;
}















