
/*	Copyright (C) G. Jespersen
	See Licence.txt.*/

#if !defined JSON_PARSE_H
#define JSON_PARSE_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Stdafx.h"
#include <afxtempl.h>
#include "KhanDB.h"


// Keyword and value pair
struct KEY_VAL
{
	KEY_VAL() { Zero(); };
	CString keyword;
	CString value;
	BOOL	valueIsNumeric;
	void Zero() 
	{ 
		keyword = _T("");
		value = _T("");
		valueIsNumeric = FALSE;

		/* Empty() is slow slow!
		keyword.Empty();
		value.Empty();
		valueIsNumeric = FALSE;*/
	};
};

// For temporary storage of topic blocks which have not
// yet been saved to the TopicTree while parsing.
class CTopicBlk
{
	public:

	CTopicBlk();
	~CTopicBlk();
	CTopicBlk(const CTopicBlk & topicBlk);

	void	operator=(const CTopicBlk & topicBlk);
	void	Copy(const CTopicBlk & topicBlk);
	void	Zero();
	BOOL	ToTreeTopic(CTopic & khanTopic, CStringArray & nodesAr);

	// Members
	int		m_nOpeningBrace;
	CString	m_description,
			m_title,
			m_extendedSlug,
			m_id,
			m_kind;
};

////////////////////////////////////////////////////////////////
// CParseJson
class CParseJson
{
public:
	CParseJson();
	~CParseJson();

public: 
	// Operations
	BOOL	ParseLibrary(CTopicTree * tree, const CString & pathIn,
						 HWND hWndDlg, int & progPos, CString & errMsg);	
private: 
	// Operations
	BOOL	IsKeyword();
	BOOL	ProcessKeyVal(CTopicTree * tree);
	void	ZeroParseVars();
	BOOL	BlockBegin();
	BOOL	BlockEnd(CTopicTree * tree, HWND hWndDlg, int & progPos);
	BOOL	GetTopicBlockData(CTopicBlk & blk);
	BOOL	IsValidTopicBlock(const CTopicBlk & blk);
	BOOL	IsValidCurrentVideo();

	BOOL	SaveVideoBlockOld();//old

private:
	// Attributes
	KEY_VAL	m_keyval;
	int		m_numBraces;
	int		m_nCharsBetween;
	int		m_braceOffset;
	UINT	m_filePos;
	UINT	m_lastBraceCloseFP;
	UINT	m_lastBraceOpenFP;

	// Run through a block
	BOOL	m_skipBlock;
	int		m_skipBlockStartBrace;

	// Attributes - key-value - see ZeroParseVars().
	CString	m_extendedSlug,				// topic
			m_dateAdded,				// video
			m_mp4, 						// video
			m_duration, 				// video
			m_description,				// both
			m_title,					// both
			m_id,						// both
			m_kind;						// both
	CVideo		m_curVideo;
	CTopicBlk	m_tempBlk;				// avoid constructor in BlockBegin()

	// Temp storage arrays.
	CArray <CTopicBlk, CTopicBlk> m_topicBlks;
	CArray <CVideo, CVideo> m_videos;

	// Error stats.
	public:
	int		m_nNodeBraceError;
	int		m_nBraceMismatch;

	////////////////////////////////////////////////////
	// Test versions for /api/v1/topictree
	// See CDataView::OnTest()
	//
	public:
	BOOL	TestParseLibrary(CTopicTree * tree, const CString & pathIn, CString & errMsg);
	
	private: // Operations
	BOOL	TestIsKeyword();
	BOOL	TestProcessKeyVal(CTopicTree * tree);
	void	TestZeroParseVars();
	BOOL	TestBlockBegin();
	BOOL	TestBlockEnd(CTopicTree * tree, HWND hWndDlg, int & progPos);
	BOOL	TestGetTopicBlockData(CTopicBlk & blk);
	BOOL	TestIsValidTopicBlock(const CTopicBlk & blk);	
	BOOL	TestIsValidCurrentVideo();	// 6/2/16

	// Test Utilities
	//
	public:
	// Format for readable.
	void	TestUtilityFormat(CString pathIn, CString pathOut);

	// Print Sluga
	void	TestUtilityPrintSlugs(CString pathOut, BOOL sort);

	// Array for printing slugs
	CSortStrArray m_testSortStrAr; // defined in KhanDB.cpp
};

#endif //JSON_PARSE_H

