
/*	Copyright (C) G. Jespersen
	See Licence.txt.*/

#if !defined KHAN_JSON_H
#define KHAN_JSON_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Stdafx.h"
#include <afxtempl.h>
#include "KhanDB.h"


// Keyword and value pair
struct KEYVAL
{
	KEYVAL() { Zero(); };
	CString keyword;
	CString value;
	BOOL	valueIsNumeric;
	void Zero() 
	{ 
		keyword = _T("");
		value = _T("");
		valueIsNumeric = FALSE;
	};
};

////////////////////////////////////////////////////////////////
// CKhanJson
class CKhanJson
{
public:
	CKhanJson();
	~CKhanJson();

public:
	BOOL	ParseLibrary(CTopicTree * tree, const CString & pathIn,
						 HWND hWndDlg, int & progPos, CString & errMsg);
private:
	BOOL	AddCurrentTopic(CTopicTree * tree, HWND hWndDlg, int & progPos);
	BOOL	AddCurrentVideo();
	BOOL	IsKeyword();
	BOOL	ProcessKeyVal(CTopicTree * tree);
	BOOL	SetTopicNodes();	// v0.9.1
	void	ZeroCurrentTopic();

private:

	// Vars for proccessing.
	CTopic	m_curTopic;
	CVideo	m_curVideo;
	KEYVAL	m_keyval;

	int		m_numBraces;
	int		m_nVidBraces;
	int		m_nTopicBraces;
	int		m_nCharsBetween;
	BOOL	m_inVideoBlock;
	BOOL	m_inTopicBlock;
	CStringArray m_nodesAr;	// v0.9.1

	// v0.8.9
	CString m_dateAdded, m_description, m_mp4, m_duration, m_title,
			m_standaloneTitle, m_extendedSlug;

	// Error stats v0.9.0
public:
	int		m_nErrorTopics;	// v0.9.1 not used
	int		m_nErrorVideos;
	int		m_nErrorBraces;


	////////////////////////////////////////////////////
	// Test versions for /api/v1/topictree
	//				 and /api/v1/playlists/library/list
public:
	BOOL	TestParseLibrary(CTopicTree * tree, const CString & pathIn);
	BOOL	TestAddCurrentTopic(CTopicTree * tree, HWND hWndDlg, int & progPos);
	BOOL	TestAddCurrentVideo();
	BOOL	TestIsKeyword();
	BOOL	TestProcessKeyVal(CTopicTree * tree);
	void	TestZeroCurrentTopic();	
	BOOL	TestSetTopicNodes(); // v0.9.1

	// Format for readable.
	void	TestFormat(CString pathIn, CString pathOut);

	// Print Sluga
	void	TestPrintSlugs(CString pathOut, BOOL sort);

	// Array for printing slugs
	CSortStrArray m_sortStrAr;
};


#endif //KHAN_JSON_H

